# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..base import Merge


def test_Merge_inputs():
    input_map = dict(
        axis=dict(
            usedefault=True,
        ),
        no_flatten=dict(
            usedefault=True,
        ),
        ravel_inputs=dict(
            usedefault=True,
        ),
    )
    inputs = Merge.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Merge_outputs():
    output_map = dict(
        out=dict(),
    )
    outputs = Merge.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



"""Build a file URL."""
import os
import inspect
import subprocess

REVISION_CMD = "git rev-parse --short HEAD"


def _get_git_revision():
    # Comes from scikit-learn
    # https://github.com/scikit-learn/scikit-learn/blob/master/doc/sphinxext/github_link.py
    try:
        revision = subprocess.check_output(REVISION_CMD.split()).strip()
    except (subprocess.CalledProcessError, OSError):
        return None
    return revision.decode("utf-8")


def get_url(obj):
    """Return local or remote url for an object."""
    filename = inspect.getsourcefile(obj)
    uri = "file://%s" % filename
    revision = _get_git_revision()
    if revision is not None:
        shortfile = os.path.join("nipype", filename.split("nipype/")[-1])
        uri = "http://github.com/nipy/nipype/blob/%s/%s" % (revision, shortfile)
    lines, lstart = inspect.getsourcelines(obj)
    lend = len(lines) + lstart
    return "%s#L%d-L%d" % (uri, lstart, lend)



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import Undump


def test_Undump_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        coordinates_specification=dict(
            argstr="-%s",
        ),
        datatype=dict(
            argstr="-datum %s",
        ),
        default_value=dict(
            argstr="-dval %f",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fill_value=dict(
            argstr="-fval %f",
        ),
        head_only=dict(
            argstr="-head_only",
        ),
        in_file=dict(
            argstr="-master %s",
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        mask_file=dict(
            argstr="-mask %s",
            extensions=None,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        orient=dict(
            argstr="-orient %s",
        ),
        out_file=dict(
            argstr="-prefix %s",
            extensions=None,
            name_source="in_file",
        ),
        outputtype=dict(),
        srad=dict(
            argstr="-srad %f",
        ),
    )
    inputs = Undump.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Undump_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = Undump.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import BrickStat


def test_BrickStat_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        mask=dict(
            argstr="-mask %s",
            extensions=None,
            position=2,
        ),
        max=dict(
            argstr="-max",
        ),
        mean=dict(
            argstr="-mean",
        ),
        min=dict(
            argstr="-min",
            position=1,
        ),
        percentile=dict(
            argstr="-percentile %.3f %.3f %.3f",
        ),
        slow=dict(
            argstr="-slow",
        ),
        sum=dict(
            argstr="-sum",
        ),
        var=dict(
            argstr="-var",
        ),
    )
    inputs = BrickStat.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_BrickStat_outputs():
    output_map = dict(
        min_val=dict(),
    )
    outputs = BrickStat.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..petsurfer import GTMPVC


def test_GTMPVC_inputs():
    input_map = dict(
        X=dict(
            argstr="--X",
        ),
        X0=dict(
            argstr="--X0",
        ),
        args=dict(
            argstr="%s",
        ),
        auto_mask=dict(
            argstr="--auto-mask %f %f",
        ),
        beta=dict(
            argstr="--beta",
        ),
        color_table_file=dict(
            argstr="--ctab %s",
            extensions=None,
            xor=("color_table_file", "default_color_table"),
        ),
        contrast=dict(
            argstr="--C %s...",
        ),
        default_color_table=dict(
            argstr="--ctab-default",
            xor=("color_table_file", "default_color_table"),
        ),
        default_seg_merge=dict(
            argstr="--default-seg-merge",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        frame=dict(
            argstr="--frame %i",
        ),
        in_file=dict(
            argstr="--i %s",
            copyfile=False,
            extensions=None,
            mandatory=True,
        ),
        km_hb=dict(
            argstr="--km-hb %s...",
        ),
        km_ref=dict(
            argstr="--km-ref %s...",
        ),
        lat=dict(
            argstr="--lat",
        ),
        mask_file=dict(
            argstr="--mask %s",
            extensions=None,
        ),
        merge_cblum_wm_gyri=dict(
            argstr="--merge-cblum-wm-gyri",
        ),
        merge_hypos=dict(
            argstr="--merge-hypos",
        ),
        mg=dict(
            argstr="--mg %g %s",
        ),
        mg_ref_cerebral_wm=dict(
            argstr="--mg-ref-cerebral-wm",
        ),
        mg_ref_lobes_wm=dict(
            argstr="--mg-ref-lobes-wm",
        ),
        mgx=dict(
            argstr="--mgx %f",
        ),
        no_pvc=dict(
            argstr="--no-pvc",
        ),
        no_reduce_fov=dict(
            argstr="--no-reduce-fov",
        ),
        no_rescale=dict(
            argstr="--no-rescale",
        ),
        no_tfe=dict(
            argstr="--no-tfe",
        ),
        num_threads=dict(
            argstr="--threads %i",
        ),
        opt_brain=dict(
            argstr="--opt-brain",
        ),
        opt_seg_merge=dict(
            argstr="--opt-seg-merge",
        ),
        opt_tol=dict(
            argstr="--opt-tol %i %f %f",
        ),
        optimization_schema=dict(
            argstr="--opt %s",
        ),
        psf=dict(
            argstr="--psf %f",
        ),
        psf_col=dict(
            argstr="--psf-col %f",
        ),
        psf_row=dict(
            argstr="--psf-row %f",
        ),
        psf_slice=dict(
            argstr="--psf-slice %f",
        ),
        pvc_dir=dict(
            argstr="--o %s",
            genfile=True,
        ),
        rbv=dict(
            argstr="--rbv",
            requires=["subjects_dir"],
        ),
        rbv_res=dict(
            argstr="--rbv-res %f",
        ),
        reduce_fox_eqodd=dict(
            argstr="--reduce-fox-eqodd",
        ),
        reg_file=dict(
            argstr="--reg %s",
            extensions=None,
            mandatory=True,
            xor=["reg_file", "regheader", "reg_identity"],
        ),
        reg_identity=dict(
            argstr="--reg-identity",
            mandatory=True,
            xor=["reg_file", "regheader", "reg_identity"],
        ),
        regheader=dict(
            argstr="--regheader",
            mandatory=True,
            xor=["reg_file", "regheader", "reg_identity"],
        ),
        replace=dict(
            argstr="--replace %i %i",
        ),
        rescale=dict(
            argstr="--rescale %s...",
        ),
        save_eres=dict(
            argstr="--save-eres",
        ),
        save_input=dict(
            argstr="--save-input",
        ),
        save_yhat=dict(
            argstr="--save-yhat",
            xor=["save_yhat_with_noise"],
        ),
        save_yhat0=dict(
            argstr="--save-yhat0",
        ),
        save_yhat_full_fov=dict(
            argstr="--save-yhat-full-fov",
        ),
        save_yhat_with_noise=dict(
            argstr="--save-yhat-with-noise %i %i",
            xor=["save_yhat"],
        ),
        scale_refval=dict(
            argstr="--scale-refval %f",
        ),
        segmentation=dict(
            argstr="--seg %s",
            extensions=None,
            mandatory=True,
        ),
        steady_state_params=dict(
            argstr="--ss %f %f %f",
        ),
        subjects_dir=dict(),
        tissue_fraction_resolution=dict(
            argstr="--segpvfres %f",
        ),
        tt_reduce=dict(
            argstr="--tt-reduce",
        ),
        tt_update=dict(
            argstr="--tt-update",
        ),
        y=dict(
            argstr="--y",
        ),
    )
    inputs = GTMPVC.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_GTMPVC_outputs():
    output_map = dict(
        gtm_file=dict(
            extensions=None,
        ),
        gtm_stats=dict(
            extensions=None,
        ),
        hb_dat=dict(
            extensions=None,
        ),
        hb_nifti=dict(
            extensions=None,
        ),
        input_file=dict(
            extensions=None,
        ),
        mgx_ctxgm=dict(
            extensions=None,
        ),
        mgx_gm=dict(
            extensions=None,
        ),
        mgx_subctxgm=dict(
            extensions=None,
        ),
        nopvc_file=dict(
            extensions=None,
        ),
        opt_params=dict(
            extensions=None,
        ),
        pvc_dir=dict(),
        rbv=dict(
            extensions=None,
        ),
        ref_file=dict(
            extensions=None,
        ),
        reg_anat2pet=dict(
            extensions=None,
        ),
        reg_anat2rbvpet=dict(
            extensions=None,
        ),
        reg_pet2anat=dict(
            extensions=None,
        ),
        reg_rbvpet2anat=dict(
            extensions=None,
        ),
        yhat=dict(
            extensions=None,
        ),
        yhat0=dict(
            extensions=None,
        ),
        yhat_full_fov=dict(
            extensions=None,
        ),
        yhat_with_noise=dict(
            extensions=None,
        ),
    )
    outputs = GTMPVC.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""The freesurfer module provides basic functions for interfacing with
   freesurfer tools.
"""

import os

from ...utils.filemanip import fname_presuffix, split_filename
from ..base import (
    TraitedSpec,
    File,
    traits,
    InputMultiPath,
    OutputMultiPath,
    Directory,
    isdefined,
)
from .base import FSCommand, FSTraitedSpec
from .utils import copy2subjdir

__docformat__ = "restructuredtext"


class MRISPreprocInputSpec(FSTraitedSpec):
    out_file = File(argstr="--out %s", genfile=True, desc="output filename")
    target = traits.Str(
        argstr="--target %s", mandatory=True, desc="target subject name"
    )
    hemi = traits.Enum(
        "lh",
        "rh",
        argstr="--hemi %s",
        mandatory=True,
        desc="hemisphere for source and target",
    )
    surf_measure = traits.Str(
        argstr="--meas %s",
        xor=("surf_measure", "surf_measure_file", "surf_area"),
        desc="Use subject/surf/hemi.surf_measure as input",
    )
    surf_area = traits.Str(
        argstr="--area %s",
        xor=("surf_measure", "surf_measure_file", "surf_area"),
        desc="Extract vertex area from subject/surf/hemi.surfname to use as input.",
    )
    subjects = traits.List(
        argstr="--s %s...",
        xor=("subjects", "fsgd_file", "subject_file"),
        desc="subjects from who measures are calculated",
    )
    fsgd_file = File(
        exists=True,
        argstr="--fsgd %s",
        xor=("subjects", "fsgd_file", "subject_file"),
        desc="specify subjects using fsgd file",
    )
    subject_file = File(
        exists=True,
        argstr="--f %s",
        xor=("subjects", "fsgd_file", "subject_file"),
        desc="file specifying subjects separated by white space",
    )
    surf_measure_file = InputMultiPath(
        File(exists=True),
        argstr="--is %s...",
        xor=("surf_measure", "surf_measure_file", "surf_area"),
        desc="file alternative to surfmeas, still requires list of subjects",
    )
    source_format = traits.Str(argstr="--srcfmt %s", desc="source format")
    surf_dir = traits.Str(
        argstr="--surfdir %s", desc="alternative directory (instead of surf)"
    )
    vol_measure_file = InputMultiPath(
        traits.Tuple(File(exists=True), File(exists=True)),
        argstr="--iv %s %s...",
        desc="list of volume measure and reg file tuples",
    )
    proj_frac = traits.Float(
        argstr="--projfrac %s", desc="projection fraction for vol2surf"
    )
    fwhm = traits.Float(
        argstr="--fwhm %f",
        xor=["num_iters"],
        desc="smooth by fwhm mm on the target surface",
    )
    num_iters = traits.Int(
        argstr="--niters %d",
        xor=["fwhm"],
        desc="niters : smooth by niters on the target surface",
    )
    fwhm_source = traits.Float(
        argstr="--fwhm-src %f",
        xor=["num_iters_source"],
        desc="smooth by fwhm mm on the source surface",
    )
    num_iters_source = traits.Int(
        argstr="--niterssrc %d",
        xor=["fwhm_source"],
        desc="niters : smooth by niters on the source surface",
    )
    smooth_cortex_only = traits.Bool(
        argstr="--smooth-cortex-only",
        desc="only smooth cortex (ie, exclude medial wall)",
    )


class MRISPreprocOutputSpec(TraitedSpec):
    out_file = File(desc="preprocessed output file")


class MRISPreproc(FSCommand):
    """Use FreeSurfer mris_preproc to prepare a group of contrasts for
    a second level analysis

    Examples
    --------
    >>> preproc = MRISPreproc()
    >>> preproc.inputs.target = 'fsaverage'
    >>> preproc.inputs.hemi = 'lh'
    >>> preproc.inputs.vol_measure_file = [('cont1.nii', 'register.dat'), \
                                           ('cont1a.nii', 'register.dat')]
    >>> preproc.inputs.out_file = 'concatenated_file.mgz'
    >>> preproc.cmdline
    'mris_preproc --hemi lh --out concatenated_file.mgz --target fsaverage --iv cont1.nii register.dat --iv cont1a.nii register.dat'

    """

    _cmd = "mris_preproc"
    input_spec = MRISPreprocInputSpec
    output_spec = MRISPreprocOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outfile = self.inputs.out_file
        outputs["out_file"] = outfile
        if not isdefined(outfile):
            outputs["out_file"] = os.path.join(
                os.getcwd(), "concat_%s_%s.mgz" % (self.inputs.hemi, self.inputs.target)
            )
        return outputs

    def _gen_filename(self, name):
        if name == "out_file":
            return self._list_outputs()[name]
        return None


class MRISPreprocReconAllInputSpec(MRISPreprocInputSpec):
    surf_measure_file = File(
        exists=True,
        argstr="--meas %s",
        xor=("surf_measure", "surf_measure_file", "surf_area"),
        desc="file necessary for surfmeas",
    )
    surfreg_files = InputMultiPath(
        File(exists=True),
        argstr="--surfreg %s",
        requires=["lh_surfreg_target", "rh_surfreg_target"],
        desc="lh and rh input surface registration files",
    )
    lh_surfreg_target = File(
        desc="Implicit target surface registration file", requires=["surfreg_files"]
    )
    rh_surfreg_target = File(
        desc="Implicit target surface registration file", requires=["surfreg_files"]
    )
    subject_id = traits.String(
        "subject_id",
        argstr="--s %s",
        usedefault=True,
        xor=("subjects", "fsgd_file", "subject_file", "subject_id"),
        desc="subject from whom measures are calculated",
    )
    copy_inputs = traits.Bool(
        desc="If running as a node, set this to True "
        "this will copy some implicit inputs to the "
        "node directory."
    )


class MRISPreprocReconAll(MRISPreproc):
    """Extends MRISPreproc to allow it to be used in a recon-all workflow

    Examples
    --------
    >>> preproc = MRISPreprocReconAll()
    >>> preproc.inputs.target = 'fsaverage'
    >>> preproc.inputs.hemi = 'lh'
    >>> preproc.inputs.vol_measure_file = [('cont1.nii', 'register.dat'), \
                                           ('cont1a.nii', 'register.dat')]
    >>> preproc.inputs.out_file = 'concatenated_file.mgz'
    >>> preproc.cmdline
    'mris_preproc --hemi lh --out concatenated_file.mgz --s subject_id --target fsaverage --iv cont1.nii register.dat --iv cont1a.nii register.dat'

    """

    input_spec = MRISPreprocReconAllInputSpec

    def run(self, **inputs):
        if self.inputs.copy_inputs:
            self.inputs.subjects_dir = os.getcwd()
            if "subjects_dir" in inputs:
                inputs["subjects_dir"] = self.inputs.subjects_dir
            if isdefined(self.inputs.surf_dir):
                folder = self.inputs.surf_dir
            else:
                folder = "surf"
            if isdefined(self.inputs.surfreg_files):
                for surfreg in self.inputs.surfreg_files:
                    basename = os.path.basename(surfreg)
                    copy2subjdir(self, surfreg, folder, basename)
                    if basename.startswith("lh."):
                        copy2subjdir(
                            self,
                            self.inputs.lh_surfreg_target,
                            folder,
                            basename,
                            subject_id=self.inputs.target,
                        )
                    else:
                        copy2subjdir(
                            self,
                            self.inputs.rh_surfreg_target,
                            folder,
                            basename,
                            subject_id=self.inputs.target,
                        )

            if isdefined(self.inputs.surf_measure_file):
                copy2subjdir(self, self.inputs.surf_measure_file, folder)

        return super(MRISPreprocReconAll, self).run(**inputs)

    def _format_arg(self, name, spec, value):
        # mris_preproc looks for these files in the surf dir
        if name == "surfreg_files":
            basename = os.path.basename(value[0])
            return spec.argstr % basename.lstrip("rh.").lstrip("lh.")
        if name == "surf_measure_file":
            basename = os.path.basename(value)
            return spec.argstr % basename.lstrip("rh.").lstrip("lh.")
        return super(MRISPreprocReconAll, self)._format_arg(name, spec, value)


class GLMFitInputSpec(FSTraitedSpec):
    glm_dir = traits.Str(argstr="--glmdir %s", desc="save outputs to dir", genfile=True)
    in_file = File(
        desc="input 4D file", argstr="--y %s", mandatory=True, copyfile=False
    )
    _design_xor = ("fsgd", "design", "one_sample")
    fsgd = traits.Tuple(
        File(exists=True),
        traits.Enum("doss", "dods"),
        argstr="--fsgd %s %s",
        xor=_design_xor,
        desc="freesurfer descriptor file",
    )
    design = File(
        exists=True, argstr="--X %s", xor=_design_xor, desc="design matrix file"
    )
    contrast = InputMultiPath(
        File(exists=True), argstr="--C %s...", desc="contrast file"
    )

    one_sample = traits.Bool(
        argstr="--osgm",
        xor=("one_sample", "fsgd", "design", "contrast"),
        desc="construct X and C as a one-sample group mean",
    )
    no_contrast_ok = traits.Bool(
        argstr="--no-contrasts-ok", desc="do not fail if no contrasts specified"
    )
    per_voxel_reg = InputMultiPath(
        File(exists=True), argstr="--pvr %s...", desc="per-voxel regressors"
    )
    self_reg = traits.Tuple(
        traits.Int,
        traits.Int,
        traits.Int,
        argstr="--selfreg %d %d %d",
        desc="self-regressor from index col row slice",
    )
    weighted_ls = File(
        exists=True,
        argstr="--wls %s",
        xor=("weight_file", "weight_inv", "weight_sqrt"),
        desc="weighted least squares",
    )
    fixed_fx_var = File(
        exists=True, argstr="--yffxvar %s", desc="for fixed effects analysis"
    )
    fixed_fx_dof = traits.Int(
        argstr="--ffxdof %d",
        xor=["fixed_fx_dof_file"],
        desc="dof for fixed effects analysis",
    )
    fixed_fx_dof_file = File(
        argstr="--ffxdofdat %d",
        xor=["fixed_fx_dof"],
        desc="text file with dof for fixed effects analysis",
    )
    weight_file = File(
        exists=True, xor=["weighted_ls"], desc="weight for each input at each voxel"
    )
    weight_inv = traits.Bool(
        argstr="--w-inv", desc="invert weights", xor=["weighted_ls"]
    )
    weight_sqrt = traits.Bool(
        argstr="--w-sqrt", desc="sqrt of weights", xor=["weighted_ls"]
    )
    fwhm = traits.Range(low=0.0, argstr="--fwhm %f", desc="smooth input by fwhm")
    var_fwhm = traits.Range(
        low=0.0, argstr="--var-fwhm %f", desc="smooth variance by fwhm"
    )
    no_mask_smooth = traits.Bool(
        argstr="--no-mask-smooth", desc="do not mask when smoothing"
    )
    no_est_fwhm = traits.Bool(
        argstr="--no-est-fwhm", desc="turn off FWHM output estimation"
    )
    mask_file = File(exists=True, argstr="--mask %s", desc="binary mask")
    label_file = File(
        exists=True,
        argstr="--label %s",
        xor=["cortex"],
        desc="use label as mask, surfaces only",
    )
    cortex = traits.Bool(
        argstr="--cortex",
        xor=["label_file"],
        desc="use subjects ?h.cortex.label as label",
    )
    invert_mask = traits.Bool(argstr="--mask-inv", desc="invert mask")
    prune = traits.Bool(
        argstr="--prune",
        desc="remove voxels that do not have a non-zero value at each frame (def)",
    )
    no_prune = traits.Bool(
        argstr="--no-prune", xor=["prunethresh"], desc="do not prune"
    )
    prune_thresh = traits.Float(
        argstr="--prune_thr %f",
        xor=["noprune"],
        desc="prune threshold. Default is FLT_MIN",
    )
    compute_log_y = traits.Bool(
        argstr="--logy", desc="compute natural log of y prior to analysis"
    )
    save_estimate = traits.Bool(
        argstr="--yhat-save", desc="save signal estimate (yhat)"
    )
    save_residual = traits.Bool(argstr="--eres-save", desc="save residual error (eres)")
    save_res_corr_mtx = traits.Bool(
        argstr="--eres-scm",
        desc="save residual error spatial correlation matrix (eres.scm). Big!",
    )
    surf = traits.Bool(
        argstr="--surf %s %s %s",
        requires=["subject_id", "hemi"],
        desc="analysis is on a surface mesh",
    )
    subject_id = traits.Str(desc="subject id for surface geometry")
    hemi = traits.Enum("lh", "rh", desc="surface hemisphere")
    surf_geo = traits.Str(
        "white", usedefault=True, desc="surface geometry name (e.g. white, pial)"
    )
    simulation = traits.Tuple(
        traits.Enum("perm", "mc-full", "mc-z"),
        traits.Int(min=1),
        traits.Float,
        traits.Str,
        argstr="--sim %s %d %f %s",
        desc="nulltype nsim thresh csdbasename",
    )
    sim_sign = traits.Enum(
        "abs", "pos", "neg", argstr="--sim-sign %s", desc="abs, pos, or neg"
    )
    uniform = traits.Tuple(
        traits.Float,
        traits.Float,
        argstr="--uniform %f %f",
        desc="use uniform distribution instead of gaussian",
    )
    pca = traits.Bool(argstr="--pca", desc="perform pca/svd analysis on residual")
    calc_AR1 = traits.Bool(
        argstr="--tar1", desc="compute and save temporal AR1 of residual"
    )
    save_cond = traits.Bool(
        argstr="--save-cond", desc="flag to save design matrix condition at each voxel"
    )
    vox_dump = traits.Tuple(
        traits.Int,
        traits.Int,
        traits.Int,
        argstr="--voxdump %d %d %d",
        desc="dump voxel GLM and exit",
    )
    seed = traits.Int(argstr="--seed %d", desc="used for synthesizing noise")
    synth = traits.Bool(argstr="--synth", desc="replace input with gaussian")
    resynth_test = traits.Int(argstr="--resynthtest %d", desc="test GLM by resynthsis")
    profile = traits.Int(argstr="--profile %d", desc="niters : test speed")
    mrtm1 = traits.Tuple(
        File(exists=True),
        File(exists=True),
        argstr="--mrtm1 %s %s",
        desc="RefTac TimeSec : perform MRTM1 kinetic modeling",
    )
    mrtm2 = traits.Tuple(
        File(exists=True),
        File(exists=True),
        traits.Float,
        argstr="--mrtm2 %s %s %f",
        desc="RefTac TimeSec k2prime : perform MRTM2 kinetic modeling",
    )
    logan = traits.Tuple(
        File(exists=True),
        File(exists=True),
        traits.Float,
        argstr="--logan %s %s %f",
        desc="RefTac TimeSec tstar   : perform Logan kinetic modeling",
    )
    force_perm = traits.Bool(
        argstr="--perm-force",
        desc="force perumtation test, even when design matrix is not orthog",
    )
    diag = traits.Int(argstr="--diag %d", desc="Gdiag_no : set diagnositc level")
    diag_cluster = traits.Bool(
        argstr="--diag-cluster", desc="save sig volume and exit from first sim loop"
    )
    debug = traits.Bool(argstr="--debug", desc="turn on debugging")
    check_opts = traits.Bool(
        argstr="--checkopts", desc="don't run anything, just check options and exit"
    )
    allow_repeated_subjects = traits.Bool(
        argstr="--allowsubjrep",
        desc="allow subject names to repeat in the fsgd file (must appear before --fsgd",
    )
    allow_ill_cond = traits.Bool(
        argstr="--illcond", desc="allow ill-conditioned design matrices"
    )
    sim_done_file = File(
        argstr="--sim-done %s", desc="create file when simulation finished"
    )
    _ext_xor = ['nii', 'nii_gz']
    nii = traits.Bool(argstr='--nii', desc='save outputs as nii', xor=_ext_xor)
    nii_gz = traits.Bool(argstr='--nii.gz', desc='save outputs as nii.gz', xor=_ext_xor)


class GLMFitOutputSpec(TraitedSpec):

    glm_dir = Directory(exists=True, desc="output directory")
    beta_file = File(exists=True, desc="map of regression coefficients")
    error_file = File(desc="map of residual error")
    error_var_file = File(desc="map of residual error variance")
    error_stddev_file = File(desc="map of residual error standard deviation")
    estimate_file = File(desc="map of the estimated Y values")
    mask_file = File(desc="map of the mask used in the analysis")
    fwhm_file = File(desc="text file with estimated smoothness")
    dof_file = File(desc="text file with effective degrees-of-freedom for the analysis")
    gamma_file = OutputMultiPath(desc="map of contrast of regression coefficients")
    gamma_var_file = OutputMultiPath(desc="map of regression contrast variance")
    sig_file = OutputMultiPath(desc="map of F-test significance (in -log10p)")
    ftest_file = OutputMultiPath(desc="map of test statistic values")
    spatial_eigenvectors = File(desc="map of spatial eigenvectors from residual PCA")
    frame_eigenvectors = File(desc="matrix of frame eigenvectors from residual PCA")
    singular_values = File(desc="matrix singular values from residual PCA")
    svd_stats_file = File(desc="text file summarizing the residual PCA")
    k2p_file = File(desc="estimate of k2p parameter")
    bp_file = File(desc="Binding potential estimates")


class GLMFit(FSCommand):
    """Use FreeSurfer's mri_glmfit to specify and estimate a general linear model.

    Examples
    --------
    >>> glmfit = GLMFit()
    >>> glmfit.inputs.in_file = 'functional.nii'
    >>> glmfit.inputs.one_sample = True
    >>> glmfit.cmdline == 'mri_glmfit --glmdir %s --y functional.nii --osgm'%os.getcwd()
    True

    """

    _cmd = "mri_glmfit"
    input_spec = GLMFitInputSpec
    output_spec = GLMFitOutputSpec

    def _format_arg(self, name, spec, value):
        if name == "surf":
            _si = self.inputs
            return spec.argstr % (_si.subject_id, _si.hemi, _si.surf_geo)
        return super(GLMFit, self)._format_arg(name, spec, value)

    def _list_outputs(self):
        outputs = self.output_spec().get()
        # Get the top-level output directory
        if not isdefined(self.inputs.glm_dir):
            glmdir = os.getcwd()
        else:
            glmdir = os.path.abspath(self.inputs.glm_dir)
        outputs["glm_dir"] = glmdir

        if isdefined(self.inputs.nii_gz):
            ext = 'nii.gz'
        elif isdefined(self.inputs.nii):
            ext = 'nii'
        else:
            ext = 'mgh'

        # Assign the output files that always get created
        outputs["beta_file"] = os.path.join(glmdir, f"beta.{ext}")
        outputs["error_var_file"] = os.path.join(glmdir, f"rvar.{ext}")
        outputs["error_stddev_file"] = os.path.join(glmdir, f"rstd.{ext}")
        outputs["mask_file"] = os.path.join(glmdir, f"mask.{ext}")
        outputs["fwhm_file"] = os.path.join(glmdir, "fwhm.dat")
        outputs["dof_file"] = os.path.join(glmdir, "dof.dat")
        # Assign the conditional outputs
        if self.inputs.save_residual:
            outputs["error_file"] = os.path.join(glmdir, f"eres.{ext}")
        if self.inputs.save_estimate:
            outputs["estimate_file"] = os.path.join(glmdir, f"yhat.{ext}")
        if any((self.inputs.mrtm1, self.inputs.mrtm2, self.inputs.logan)):
            outputs["bp_file"] = os.path.join(glmdir, f"bp.{ext}")
        if self.inputs.mrtm1:
            outputs["k2p_file"] = os.path.join(glmdir, "k2prime.dat")

        # Get the contrast directory name(s)
        contrasts = []
        if isdefined(self.inputs.contrast):
            for c in self.inputs.contrast:
                if split_filename(c)[2] in [".mat", ".dat", ".mtx", ".con"]:
                    contrasts.append(split_filename(c)[1])
                else:
                    contrasts.append(os.path.split(c)[1])
        elif isdefined(self.inputs.one_sample) and self.inputs.one_sample:
            contrasts = ["osgm"]

        # Add in the contrast images
        outputs["sig_file"] = [os.path.join(glmdir, c, f"sig.{ext}") for c in contrasts]
        outputs["ftest_file"] = [os.path.join(glmdir, c, f"F.{ext}") for c in contrasts]
        outputs["gamma_file"] = [
            os.path.join(glmdir, c, f"gamma.{ext}") for c in contrasts
        ]
        outputs["gamma_var_file"] = [
            os.path.join(glmdir, c, f"gammavar.{ext}") for c in contrasts
        ]

        # Add in the PCA results, if relevant
        if isdefined(self.inputs.pca) and self.inputs.pca:
            pcadir = os.path.join(glmdir, "pca-eres")
            outputs["spatial_eigenvectors"] = os.path.join(pcadir, f"v.{ext}")
            outputs["frame_eigenvectors"] = os.path.join(pcadir, "u.mtx")
            outputs["singluar_values"] = os.path.join(pcadir, "sdiag.mat")
            outputs["svd_stats_file"] = os.path.join(pcadir, "stats.dat")

        return outputs

    def _gen_filename(self, name):
        if name == "glm_dir":
            return os.getcwd()
        return None


class OneSampleTTest(GLMFit):
    def __init__(self, **kwargs):
        super(OneSampleTTest, self).__init__(**kwargs)
        self.inputs.one_sample = True


class BinarizeInputSpec(FSTraitedSpec):
    in_file = File(
        exists=True,
        argstr="--i %s",
        mandatory=True,
        copyfile=False,
        desc="input volume",
    )
    min = traits.Float(argstr="--min %f", xor=["wm_ven_csf"], desc="min thresh")
    max = traits.Float(argstr="--max %f", xor=["wm_ven_csf"], desc="max thresh")
    rmin = traits.Float(argstr="--rmin %f", desc="compute min based on rmin*globalmean")
    rmax = traits.Float(argstr="--rmax %f", desc="compute max based on rmax*globalmean")
    match = traits.List(
        traits.Int, argstr="--match %d...", desc="match instead of threshold"
    )
    wm = traits.Bool(
        argstr="--wm", desc="set match vals to 2 and 41 (aseg for cerebral WM)"
    )
    ventricles = traits.Bool(
        argstr="--ventricles",
        desc="set match vals those for aseg ventricles+choroid (not 4th)",
    )
    wm_ven_csf = traits.Bool(
        argstr="--wm+vcsf",
        xor=["min", "max"],
        desc="WM and ventricular CSF, including choroid (not 4th)",
    )
    binary_file = File(argstr="--o %s", genfile=True, desc="binary output volume")
    out_type = traits.Enum("nii", "nii.gz", "mgz", argstr="", desc="output file type")
    count_file = traits.Either(
        traits.Bool,
        File,
        argstr="--count %s",
        desc="save number of hits in ascii file (hits, ntotvox, pct)",
    )
    bin_val = traits.Int(
        argstr="--binval %d", desc="set vox within thresh to val (default is 1)"
    )
    bin_val_not = traits.Int(
        argstr="--binvalnot %d", desc="set vox outside range to val (default is 0)"
    )
    invert = traits.Bool(argstr="--inv", desc="set binval=0, binvalnot=1")
    frame_no = traits.Int(
        argstr="--frame %s", desc="use 0-based frame of input (default is 0)"
    )
    merge_file = File(exists=True, argstr="--merge %s", desc="merge with mergevol")
    mask_file = File(exists=True, argstr="--mask maskvol", desc="must be within mask")
    mask_thresh = traits.Float(argstr="--mask-thresh %f", desc="set thresh for mask")
    abs = traits.Bool(
        argstr="--abs", desc="take abs of invol first (ie, make unsigned)"
    )
    bin_col_num = traits.Bool(
        argstr="--bincol", desc="set binarized voxel value to its column number"
    )
    zero_edges = traits.Bool(argstr="--zero-edges", desc="zero the edge voxels")
    zero_slice_edge = traits.Bool(
        argstr="--zero-slice-edges", desc="zero the edge slice voxels"
    )
    dilate = traits.Int(argstr="--dilate %d", desc="niters: dilate binarization in 3D")
    erode = traits.Int(
        argstr="--erode  %d",
        desc="nerode: erode binarization in 3D (after any dilation)",
    )
    erode2d = traits.Int(
        argstr="--erode2d %d",
        desc="nerode2d: erode binarization in 2D (after any 3D erosion)",
    )


class BinarizeOutputSpec(TraitedSpec):
    binary_file = File(exists=True, desc="binarized output volume")
    count_file = File(desc="ascii file containing number of hits")


class Binarize(FSCommand):
    """Use FreeSurfer mri_binarize to threshold an input volume

    Examples
    --------
    >>> binvol = Binarize(in_file='structural.nii', min=10, binary_file='foo_out.nii')
    >>> binvol.cmdline
    'mri_binarize --o foo_out.nii --i structural.nii --min 10.000000'

    """

    _cmd = "mri_binarize"
    input_spec = BinarizeInputSpec
    output_spec = BinarizeOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outfile = self.inputs.binary_file
        if not isdefined(outfile):
            if isdefined(self.inputs.out_type):
                outfile = fname_presuffix(
                    self.inputs.in_file,
                    newpath=os.getcwd(),
                    suffix=".".join(("_thresh", self.inputs.out_type)),
                    use_ext=False,
                )
            else:
                outfile = fname_presuffix(
                    self.inputs.in_file, newpath=os.getcwd(), suffix="_thresh"
                )
        outputs["binary_file"] = os.path.abspath(outfile)
        value = self.inputs.count_file
        if isdefined(value):
            if isinstance(value, bool):
                if value:
                    outputs["count_file"] = fname_presuffix(
                        self.inputs.in_file,
                        suffix="_count.txt",
                        newpath=os.getcwd(),
                        use_ext=False,
                    )
            else:
                outputs["count_file"] = value
        return outputs

    def _format_arg(self, name, spec, value):
        if name == "count_file":
            if isinstance(value, bool):
                fname = self._list_outputs()[name]
            else:
                fname = value
            return spec.argstr % fname
        if name == "out_type":
            return ""
        return super(Binarize, self)._format_arg(name, spec, value)

    def _gen_filename(self, name):
        if name == "binary_file":
            return self._list_outputs()[name]
        return None


class ConcatenateInputSpec(FSTraitedSpec):
    in_files = InputMultiPath(
        File(exists=True),
        desc="Individual volumes to be concatenated",
        argstr="--i %s...",
        mandatory=True,
    )
    concatenated_file = File(desc="Output volume", argstr="--o %s", genfile=True)
    sign = traits.Enum(
        "abs",
        "pos",
        "neg",
        argstr="--%s",
        desc="Take only pos or neg voxles from input, or take abs",
    )
    stats = traits.Enum(
        "sum",
        "var",
        "std",
        "max",
        "min",
        "mean",
        argstr="--%s",
        desc="Compute the sum, var, std, max, min or mean of the input volumes",
    )
    paired_stats = traits.Enum(
        "sum",
        "avg",
        "diff",
        "diff-norm",
        "diff-norm1",
        "diff-norm2",
        argstr="--paired-%s",
        desc="Compute paired sum, avg, or diff",
    )
    gmean = traits.Int(
        argstr="--gmean %d", desc="create matrix to average Ng groups, Nper=Ntot/Ng"
    )
    mean_div_n = traits.Bool(
        argstr="--mean-div-n", desc="compute mean/nframes (good for var)"
    )
    multiply_by = traits.Float(
        argstr="--mul %f", desc="Multiply input volume by some amount"
    )
    add_val = traits.Float(
        argstr="--add %f", desc="Add some amount to the input volume"
    )
    multiply_matrix_file = File(
        exists=True, argstr="--mtx %s", desc="Multiply input by an ascii matrix in file"
    )
    combine = traits.Bool(
        argstr="--combine", desc="Combine non-zero values into single frame volume"
    )
    keep_dtype = traits.Bool(
        argstr="--keep-datatype", desc="Keep voxelwise precision type (default is float"
    )
    max_bonfcor = traits.Bool(
        argstr="--max-bonfcor",
        desc="Compute max and bonferroni correct (assumes -log10(ps))",
    )
    max_index = traits.Bool(
        argstr="--max-index",
        desc="Compute the index of max voxel in concatenated volumes",
    )
    mask_file = File(exists=True, argstr="--mask %s", desc="Mask input with a volume")
    vote = traits.Bool(
        argstr="--vote",
        desc="Most frequent value at each voxel and fraction of occurances",
    )
    sort = traits.Bool(argstr="--sort", desc="Sort each voxel by ascending frame value")


class ConcatenateOutputSpec(TraitedSpec):
    concatenated_file = File(exists=True, desc="Path/name of the output volume")


class Concatenate(FSCommand):
    """Use Freesurfer mri_concat to combine several input volumes
    into one output volume.  Can concatenate by frames, or compute
    a variety of statistics on the input volumes.

    Examples
    --------
    Combine two input volumes into one volume with two frames

    >>> concat = Concatenate()
    >>> concat.inputs.in_files = ['cont1.nii', 'cont2.nii']
    >>> concat.inputs.concatenated_file = 'bar.nii'
    >>> concat.cmdline
    'mri_concat --o bar.nii --i cont1.nii --i cont2.nii'

    """

    _cmd = "mri_concat"
    input_spec = ConcatenateInputSpec
    output_spec = ConcatenateOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()

        fname = self.inputs.concatenated_file
        if not isdefined(fname):
            fname = "concat_output.nii.gz"
        outputs["concatenated_file"] = os.path.join(os.getcwd(), fname)
        return outputs

    def _gen_filename(self, name):
        if name == "concatenated_file":
            return self._list_outputs()[name]
        return None


class SegStatsInputSpec(FSTraitedSpec):
    _xor_inputs = ("segmentation_file", "annot", "surf_label")
    segmentation_file = File(
        exists=True,
        argstr="--seg %s",
        xor=_xor_inputs,
        mandatory=True,
        desc="segmentation volume path",
    )
    annot = traits.Tuple(
        traits.Str,
        traits.Enum("lh", "rh"),
        traits.Str,
        argstr="--annot %s %s %s",
        xor=_xor_inputs,
        mandatory=True,
        desc="subject hemi parc : use surface parcellation",
    )
    surf_label = traits.Tuple(
        traits.Str,
        traits.Enum("lh", "rh"),
        traits.Str,
        argstr="--slabel %s %s %s",
        xor=_xor_inputs,
        mandatory=True,
        desc="subject hemi label : use surface label",
    )
    summary_file = File(
        argstr="--sum %s",
        genfile=True,
        position=-1,
        desc="Segmentation stats summary table file",
    )
    partial_volume_file = File(
        exists=True, argstr="--pv %s", desc="Compensate for partial voluming"
    )
    in_file = File(
        exists=True,
        argstr="--i %s",
        desc="Use the segmentation to report stats on this volume",
    )
    frame = traits.Int(
        argstr="--frame %d", desc="Report stats on nth frame of input volume"
    )
    multiply = traits.Float(argstr="--mul %f", desc="multiply input by val")
    calc_snr = traits.Bool(
        argstr="--snr", desc="save mean/std as extra column in output table"
    )
    calc_power = traits.Enum(
        "sqr",
        "sqrt",
        argstr="--%s",
        desc="Compute either the sqr or the sqrt of the input",
    )
    _ctab_inputs = ("color_table_file", "default_color_table", "gca_color_table")
    color_table_file = File(
        exists=True,
        argstr="--ctab %s",
        xor=_ctab_inputs,
        desc="color table file with seg id names",
    )
    default_color_table = traits.Bool(
        argstr="--ctab-default",
        xor=_ctab_inputs,
        desc="use $FREESURFER_HOME/FreeSurferColorLUT.txt",
    )
    gca_color_table = File(
        exists=True,
        argstr="--ctab-gca %s",
        xor=_ctab_inputs,
        desc="get color table from GCA (CMA)",
    )
    segment_id = traits.List(
        argstr="--id %s...", desc="Manually specify segmentation ids"
    )
    exclude_id = traits.Int(argstr="--excludeid %d", desc="Exclude seg id from report")
    exclude_ctx_gm_wm = traits.Bool(
        argstr="--excl-ctxgmwm", desc="exclude cortical gray and white matter"
    )
    wm_vol_from_surf = traits.Bool(
        argstr="--surf-wm-vol", desc="Compute wm volume from surf"
    )
    cortex_vol_from_surf = traits.Bool(
        argstr="--surf-ctx-vol", desc="Compute cortex volume from surf"
    )
    non_empty_only = traits.Bool(
        argstr="--nonempty", desc="Only report nonempty segmentations"
    )
    empty = traits.Bool(
        argstr="--empty", desc="Report on segmentations listed in the color table"
    )
    mask_file = File(
        exists=True, argstr="--mask %s", desc="Mask volume (same size as seg"
    )
    mask_thresh = traits.Float(
        argstr="--maskthresh %f", desc="binarize mask with this threshold <0.5>"
    )
    mask_sign = traits.Enum(
        "abs",
        "pos",
        "neg",
        "--masksign %s",
        desc="Sign for mask threshold: pos, neg, or abs",
    )
    mask_frame = traits.Int(
        "--maskframe %d",
        requires=["mask_file"],
        desc="Mask with this (0 based) frame of the mask volume",
    )
    mask_invert = traits.Bool(
        argstr="--maskinvert", desc="Invert binarized mask volume"
    )
    mask_erode = traits.Int(argstr="--maskerode %d", desc="Erode mask by some amount")
    brain_vol = traits.Enum(
        "brain-vol-from-seg",
        "brainmask",
        argstr="--%s",
        desc="Compute brain volume either with ``brainmask`` or ``brain-vol-from-seg``",
    )
    brainmask_file = File(
        argstr="--brainmask %s",
        exists=True,
        desc="Load brain mask and compute the volume of the brain as the non-zero voxels in this volume",
    )
    etiv = traits.Bool(argstr="--etiv", desc="Compute ICV from talairach transform")
    etiv_only = traits.Enum(
        "etiv",
        "old-etiv",
        "--%s-only",
        desc="Compute etiv and exit.  Use ``etiv`` or ``old-etiv``",
    )
    avgwf_txt_file = traits.Either(
        traits.Bool,
        File,
        argstr="--avgwf %s",
        desc="Save average waveform into file (bool or filename)",
    )
    avgwf_file = traits.Either(
        traits.Bool,
        File,
        argstr="--avgwfvol %s",
        desc="Save as binary volume (bool or filename)",
    )
    sf_avg_file = traits.Either(
        traits.Bool, File, argstr="--sfavg %s", desc="Save mean across space and time"
    )
    vox = traits.List(
        traits.Int,
        argstr="--vox %s",
        desc="Replace seg with all 0s except at C R S (three int inputs)",
    )
    supratent = traits.Bool(argstr="--supratent", desc="Undocumented input flag")
    subcort_gm = traits.Bool(
        argstr="--subcortgray", desc="Compute volume of subcortical gray matter"
    )
    total_gray = traits.Bool(
        argstr="--totalgray", desc="Compute volume of total gray matter"
    )
    euler = traits.Bool(
        argstr="--euler",
        desc="Write out number of defect holes in orig.nofix based on the euler number",
    )
    in_intensity = File(
        argstr="--in %s --in-intensity-name %s", desc="Undocumented input norm.mgz file"
    )
    intensity_units = traits.Enum(
        "MR",
        argstr="--in-intensity-units %s",
        requires=["in_intensity"],
        desc="Intensity units",
    )


class SegStatsOutputSpec(TraitedSpec):
    summary_file = File(exists=True, desc="Segmentation summary statistics table")
    avgwf_txt_file = File(
        desc="Text file with functional statistics averaged over segs"
    )
    avgwf_file = File(desc="Volume with functional statistics averaged over segs")
    sf_avg_file = File(
        desc="Text file with func statistics averaged over segs and framss"
    )


class SegStats(FSCommand):
    """Use FreeSurfer mri_segstats for ROI analysis

    Examples
    --------
    >>> import nipype.interfaces.freesurfer as fs
    >>> ss = fs.SegStats()
    >>> ss.inputs.annot = ('PWS04', 'lh', 'aparc')
    >>> ss.inputs.in_file = 'functional.nii'
    >>> ss.inputs.subjects_dir = '.'
    >>> ss.inputs.avgwf_txt_file = 'avgwf.txt'
    >>> ss.inputs.summary_file = 'summary.stats'
    >>> ss.cmdline
    'mri_segstats --annot PWS04 lh aparc --avgwf ./avgwf.txt --i functional.nii --sum ./summary.stats'

    """

    _cmd = "mri_segstats"
    input_spec = SegStatsInputSpec
    output_spec = SegStatsOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        if isdefined(self.inputs.summary_file):
            outputs["summary_file"] = os.path.abspath(self.inputs.summary_file)
        else:
            outputs["summary_file"] = os.path.join(os.getcwd(), "summary.stats")
        suffices = dict(
            avgwf_txt_file="_avgwf.txt",
            avgwf_file="_avgwf.nii.gz",
            sf_avg_file="sfavg.txt",
        )
        if isdefined(self.inputs.segmentation_file):
            _, src = os.path.split(self.inputs.segmentation_file)
        if isdefined(self.inputs.annot):
            src = "_".join(self.inputs.annot)
        if isdefined(self.inputs.surf_label):
            src = "_".join(self.inputs.surf_label)
        for name, suffix in list(suffices.items()):
            value = getattr(self.inputs, name)
            if isdefined(value):
                if isinstance(value, bool):
                    outputs[name] = fname_presuffix(
                        src, suffix=suffix, newpath=os.getcwd(), use_ext=False
                    )
                else:
                    outputs[name] = os.path.abspath(value)
        return outputs

    def _format_arg(self, name, spec, value):
        if name in ("summary_file", "avgwf_txt_file"):
            if not isinstance(value, bool):
                if not os.path.isabs(value):
                    value = os.path.join(".", value)
        if name in ["avgwf_txt_file", "avgwf_file", "sf_avg_file"]:
            if isinstance(value, bool):
                fname = self._list_outputs()[name]
            else:
                fname = value
            return spec.argstr % fname
        elif name == "in_intensity":
            intensity_name = os.path.basename(self.inputs.in_intensity).replace(
                ".mgz", ""
            )
            return spec.argstr % (value, intensity_name)
        return super(SegStats, self)._format_arg(name, spec, value)

    def _gen_filename(self, name):
        if name == "summary_file":
            return self._list_outputs()[name]
        return None


class SegStatsReconAllInputSpec(SegStatsInputSpec):
    # recon-all input requirements
    subject_id = traits.String(
        "subject_id",
        usedefault=True,
        argstr="--subject %s",
        mandatory=True,
        desc="Subject id being processed",
    )
    # implicit
    ribbon = File(mandatory=True, exists=True, desc="Input file mri/ribbon.mgz")
    presurf_seg = File(exists=True, desc="Input segmentation volume")
    transform = File(mandatory=True, exists=True, desc="Input transform file")
    lh_orig_nofix = File(mandatory=True, exists=True, desc="Input lh.orig.nofix")
    rh_orig_nofix = File(mandatory=True, exists=True, desc="Input rh.orig.nofix")
    lh_white = File(
        mandatory=True,
        exists=True,
        desc="Input file must be <subject_id>/surf/lh.white",
    )
    rh_white = File(
        mandatory=True,
        exists=True,
        desc="Input file must be <subject_id>/surf/rh.white",
    )
    lh_pial = File(
        mandatory=True, exists=True, desc="Input file must be <subject_id>/surf/lh.pial"
    )
    rh_pial = File(
        mandatory=True, exists=True, desc="Input file must be <subject_id>/surf/rh.pial"
    )
    aseg = File(exists=True, desc="Mandatory implicit input in 5.3")
    copy_inputs = traits.Bool(
        desc="If running as a node, set this to True "
        "otherwise, this will copy the implicit inputs "
        "to the node directory."
    )


class SegStatsReconAll(SegStats):
    """
    This class inherits SegStats and modifies it for use in a recon-all workflow.
    This implementation mandates implicit inputs that SegStats.
    To ensure backwards compatability of SegStats, this class was created.

    Examples
    --------
    >>> from nipype.interfaces.freesurfer import SegStatsReconAll
    >>> segstatsreconall = SegStatsReconAll()
    >>> segstatsreconall.inputs.annot = ('PWS04', 'lh', 'aparc')
    >>> segstatsreconall.inputs.avgwf_txt_file = 'avgwf.txt'
    >>> segstatsreconall.inputs.summary_file = 'summary.stats'
    >>> segstatsreconall.inputs.subject_id = '10335'
    >>> segstatsreconall.inputs.ribbon = 'wm.mgz'
    >>> segstatsreconall.inputs.transform = 'trans.mat'
    >>> segstatsreconall.inputs.presurf_seg = 'wm.mgz'
    >>> segstatsreconall.inputs.lh_orig_nofix = 'lh.pial'
    >>> segstatsreconall.inputs.rh_orig_nofix = 'lh.pial'
    >>> segstatsreconall.inputs.lh_pial = 'lh.pial'
    >>> segstatsreconall.inputs.rh_pial = 'lh.pial'
    >>> segstatsreconall.inputs.lh_white = 'lh.pial'
    >>> segstatsreconall.inputs.rh_white = 'lh.pial'
    >>> segstatsreconall.inputs.empty = True
    >>> segstatsreconall.inputs.brain_vol = 'brain-vol-from-seg'
    >>> segstatsreconall.inputs.exclude_ctx_gm_wm = True
    >>> segstatsreconall.inputs.supratent = True
    >>> segstatsreconall.inputs.subcort_gm = True
    >>> segstatsreconall.inputs.etiv = True
    >>> segstatsreconall.inputs.wm_vol_from_surf = True
    >>> segstatsreconall.inputs.cortex_vol_from_surf = True
    >>> segstatsreconall.inputs.total_gray = True
    >>> segstatsreconall.inputs.euler = True
    >>> segstatsreconall.inputs.exclude_id = 0
    >>> segstatsreconall.cmdline
    'mri_segstats --annot PWS04 lh aparc --avgwf ./avgwf.txt --brain-vol-from-seg --surf-ctx-vol --empty --etiv --euler --excl-ctxgmwm --excludeid 0 --subcortgray --subject 10335 --supratent --totalgray --surf-wm-vol --sum ./summary.stats'

    """

    input_spec = SegStatsReconAllInputSpec
    output_spec = SegStatsOutputSpec

    def _format_arg(self, name, spec, value):
        if name == "brainmask_file":
            return spec.argstr % os.path.basename(value)
        return super(SegStatsReconAll, self)._format_arg(name, spec, value)

    def run(self, **inputs):
        if self.inputs.copy_inputs:
            self.inputs.subjects_dir = os.getcwd()
            if "subjects_dir" in inputs:
                inputs["subjects_dir"] = self.inputs.subjects_dir
            copy2subjdir(self, self.inputs.lh_orig_nofix, "surf", "lh.orig.nofix")
            copy2subjdir(self, self.inputs.rh_orig_nofix, "surf", "rh.orig.nofix")
            copy2subjdir(self, self.inputs.lh_white, "surf", "lh.white")
            copy2subjdir(self, self.inputs.rh_white, "surf", "rh.white")
            copy2subjdir(self, self.inputs.lh_pial, "surf", "lh.pial")
            copy2subjdir(self, self.inputs.rh_pial, "surf", "rh.pial")
            copy2subjdir(self, self.inputs.ribbon, "mri", "ribbon.mgz")
            copy2subjdir(self, self.inputs.presurf_seg, "mri", "aseg.presurf.mgz")
            copy2subjdir(self, self.inputs.aseg, "mri", "aseg.mgz")
            copy2subjdir(
                self,
                self.inputs.transform,
                os.path.join("mri", "transforms"),
                "talairach.xfm",
            )
            copy2subjdir(self, self.inputs.in_intensity, "mri")
            copy2subjdir(self, self.inputs.brainmask_file, "mri")
        return super(SegStatsReconAll, self).run(**inputs)


class Label2VolInputSpec(FSTraitedSpec):
    label_file = InputMultiPath(
        File(exists=True),
        argstr="--label %s...",
        xor=("label_file", "annot_file", "seg_file", "aparc_aseg"),
        copyfile=False,
        mandatory=True,
        desc="list of label files",
    )
    annot_file = File(
        exists=True,
        argstr="--annot %s",
        xor=("label_file", "annot_file", "seg_file", "aparc_aseg"),
        requires=("subject_id", "hemi"),
        mandatory=True,
        copyfile=False,
        desc="surface annotation file",
    )
    seg_file = File(
        exists=True,
        argstr="--seg %s",
        xor=("label_file", "annot_file", "seg_file", "aparc_aseg"),
        mandatory=True,
        copyfile=False,
        desc="segmentation file",
    )
    aparc_aseg = traits.Bool(
        argstr="--aparc+aseg",
        xor=("label_file", "annot_file", "seg_file", "aparc_aseg"),
        mandatory=True,
        desc="use aparc+aseg.mgz in subjectdir as seg",
    )
    template_file = File(
        exists=True, argstr="--temp %s", mandatory=True, desc="output template volume"
    )
    reg_file = File(
        exists=True,
        argstr="--reg %s",
        xor=("reg_file", "reg_header", "identity"),
        desc="tkregister style matrix VolXYZ = R*LabelXYZ",
    )
    reg_header = File(
        exists=True,
        argstr="--regheader %s",
        xor=("reg_file", "reg_header", "identity"),
        desc="label template volume",
    )
    identity = traits.Bool(
        argstr="--identity", xor=("reg_file", "reg_header", "identity"), desc="set R=I"
    )
    invert_mtx = traits.Bool(
        argstr="--invertmtx", desc="Invert the registration matrix"
    )
    fill_thresh = traits.Range(
        0.0, 1.0, argstr="--fillthresh %g", desc="thresh : between 0 and 1"
    )
    label_voxel_volume = traits.Float(
        argstr="--labvoxvol %f", desc="volume of each label point (def 1mm3)"
    )
    proj = traits.Tuple(
        traits.Enum("abs", "frac"),
        traits.Float,
        traits.Float,
        traits.Float,
        argstr="--proj %s %f %f %f",
        requires=("subject_id", "hemi"),
        desc="project along surface normal",
    )
    subject_id = traits.Str(argstr="--subject %s", desc="subject id")
    hemi = traits.Enum(
        "lh", "rh", argstr="--hemi %s", desc="hemisphere to use lh or rh"
    )
    surface = traits.Str(argstr="--surf %s", desc="use surface instead of white")
    vol_label_file = File(argstr="--o %s", genfile=True, desc="output volume")
    label_hit_file = File(
        argstr="--hits %s", desc="file with each frame is nhits for a label"
    )
    map_label_stat = File(
        argstr="--label-stat %s", desc="map the label stats field into the vol"
    )
    native_vox2ras = traits.Bool(
        argstr="--native-vox2ras",
        desc="use native vox2ras xform instead of  tkregister-style",
    )


class Label2VolOutputSpec(TraitedSpec):
    vol_label_file = File(exists=True, desc="output volume")


class Label2Vol(FSCommand):
    """Make a binary volume from a Freesurfer label

    Examples
    --------
    >>> binvol = Label2Vol(label_file='cortex.label', template_file='structural.nii', reg_file='register.dat', fill_thresh=0.5, vol_label_file='foo_out.nii')
    >>> binvol.cmdline
    'mri_label2vol --fillthresh 0.5 --label cortex.label --reg register.dat --temp structural.nii --o foo_out.nii'

    """

    _cmd = "mri_label2vol"
    input_spec = Label2VolInputSpec
    output_spec = Label2VolOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        outfile = self.inputs.vol_label_file
        if not isdefined(outfile):
            for key in ["label_file", "annot_file", "seg_file"]:
                if isdefined(getattr(self.inputs, key)):
                    path = getattr(self.inputs, key)
                    if isinstance(path, list):
                        path = path[0]
                    _, src = os.path.split(path)
            if isdefined(self.inputs.aparc_aseg):
                src = "aparc+aseg.mgz"
            outfile = fname_presuffix(
                src, suffix="_vol.nii.gz", newpath=os.getcwd(), use_ext=False
            )
        outputs["vol_label_file"] = outfile
        return outputs

    def _gen_filename(self, name):
        if name == "vol_label_file":
            return self._list_outputs()[name]
        return None


class MS_LDAInputSpec(FSTraitedSpec):
    lda_labels = traits.List(
        traits.Int(),
        argstr="-lda %s",
        mandatory=True,
        minlen=2,
        maxlen=2,
        sep=" ",
        desc="pair of class labels to optimize",
    )
    weight_file = File(
        argstr="-weight %s",
        mandatory=True,
        desc="filename for the LDA weights (input or output)",
    )
    vol_synth_file = File(
        exists=False,
        argstr="-synth %s",
        mandatory=True,
        desc=("filename for the synthesized output " "volume"),
    )
    label_file = File(
        exists=True, argstr="-label %s", desc="filename of the label volume"
    )
    mask_file = File(
        exists=True, argstr="-mask %s", desc="filename of the brain mask volume"
    )
    shift = traits.Int(
        argstr="-shift %d", desc="shift all values equal to the given value to zero"
    )
    conform = traits.Bool(
        argstr="-conform",
        desc=("Conform the input volumes (brain mask " "typically already conformed)"),
    )
    use_weights = traits.Bool(
        argstr="-W", desc=("Use the weights from a previously " "generated weight file")
    )
    images = InputMultiPath(
        File(exists=True),
        argstr="%s",
        mandatory=True,
        copyfile=False,
        desc="list of input FLASH images",
        position=-1,
    )


class MS_LDAOutputSpec(TraitedSpec):
    weight_file = File(exists=True, desc="")
    vol_synth_file = File(exists=True, desc="")


class MS_LDA(FSCommand):
    """Perform LDA reduction on the intensity space of an arbitrary # of FLASH images

    Examples
    --------
    >>> grey_label = 2
    >>> white_label = 3
    >>> zero_value = 1
    >>> optimalWeights = MS_LDA(lda_labels=[grey_label, white_label], \
                                label_file='label.mgz', weight_file='weights.txt', \
                                shift=zero_value, vol_synth_file='synth_out.mgz', \
                                conform=True, use_weights=True, \
                                images=['FLASH1.mgz', 'FLASH2.mgz', 'FLASH3.mgz'])
    >>> optimalWeights.cmdline
    'mri_ms_LDA -conform -label label.mgz -lda 2 3 -shift 1 -W -synth synth_out.mgz -weight weights.txt FLASH1.mgz FLASH2.mgz FLASH3.mgz'

    """

    _cmd = "mri_ms_LDA"
    input_spec = MS_LDAInputSpec
    output_spec = MS_LDAOutputSpec

    def _list_outputs(self):
        outputs = self._outputs().get()
        if isdefined(self.inputs.output_synth):
            outputs["vol_synth_file"] = os.path.abspath(self.inputs.output_synth)
        else:
            outputs["vol_synth_file"] = os.path.abspath(self.inputs.vol_synth_file)
        if not isdefined(self.inputs.use_weights) or self.inputs.use_weights is False:
            outputs["weight_file"] = os.path.abspath(self.inputs.weight_file)
        return outputs

    def _verify_weights_file_exists(self):
        if not os.path.exists(os.path.abspath(self.inputs.weight_file)):
            raise traits.TraitError(
                "MS_LDA: use_weights must accompany an existing weights file"
            )

    def _format_arg(self, name, spec, value):
        if name == "use_weights":
            if self.inputs.use_weights is True:
                self._verify_weights_file_exists()
            else:
                return ""
                # TODO: Fix bug when boolean values are set explicitly to false
        return super(MS_LDA, self)._format_arg(name, spec, value)

    def _gen_filename(self, name):
        pass


class Label2LabelInputSpec(FSTraitedSpec):
    hemisphere = traits.Enum(
        "lh", "rh", argstr="--hemi %s", mandatory=True, desc="Input hemisphere"
    )
    subject_id = traits.String(
        "subject_id",
        usedefault=True,
        argstr="--trgsubject %s",
        mandatory=True,
        desc="Target subject",
    )
    sphere_reg = File(
        mandatory=True, exists=True, desc="Implicit input <hemisphere>.sphere.reg"
    )
    white = File(mandatory=True, exists=True, desc="Implicit input <hemisphere>.white")
    source_sphere_reg = File(
        mandatory=True, exists=True, desc="Implicit input <hemisphere>.sphere.reg"
    )
    source_white = File(
        mandatory=True, exists=True, desc="Implicit input <hemisphere>.white"
    )
    source_label = File(
        argstr="--srclabel %s", mandatory=True, exists=True, desc="Source label"
    )
    source_subject = traits.String(
        argstr="--srcsubject %s", mandatory=True, desc="Source subject name"
    )
    # optional
    out_file = File(
        argstr="--trglabel %s",
        name_source=["source_label"],
        name_template="%s_converted",
        hash_files=False,
        keep_extension=True,
        desc="Target label",
    )
    registration_method = traits.Enum(
        "surface",
        "volume",
        usedefault=True,
        argstr="--regmethod %s",
        desc="Registration method",
    )
    copy_inputs = traits.Bool(
        desc="If running as a node, set this to True."
        + "This will copy the input files to the node "
        + "directory."
    )


class Label2LabelOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc="Output label")


class Label2Label(FSCommand):
    """
    Converts a label in one subject's space to a label
    in another subject's space using either talairach or spherical
    as an intermediate registration space.

    If a source mask is used, then the input label must have been
    created from a surface (ie, the vertex numbers are valid). The
    format can be anything supported by mri_convert or curv or paint.
    Vertices in the source label that do not meet threshold in the
    mask will be removed from the label.

    Examples
    --------
    >>> from nipype.interfaces.freesurfer import Label2Label
    >>> l2l = Label2Label()
    >>> l2l.inputs.hemisphere = 'lh'
    >>> l2l.inputs.subject_id = '10335'
    >>> l2l.inputs.sphere_reg = 'lh.pial'
    >>> l2l.inputs.white = 'lh.pial'
    >>> l2l.inputs.source_subject = 'fsaverage'
    >>> l2l.inputs.source_label = 'lh-pial.stl'
    >>> l2l.inputs.source_white = 'lh.pial'
    >>> l2l.inputs.source_sphere_reg = 'lh.pial'
    >>> l2l.cmdline
    'mri_label2label --hemi lh --trglabel lh-pial_converted.stl --regmethod surface --srclabel lh-pial.stl --srcsubject fsaverage --trgsubject 10335'
    """

    _cmd = "mri_label2label"
    input_spec = Label2LabelInputSpec
    output_spec = Label2LabelOutputSpec

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs["out_file"] = os.path.join(
            self.inputs.subjects_dir,
            self.inputs.subject_id,
            "label",
            self.inputs.out_file,
        )
        return outputs

    def run(self, **inputs):
        if self.inputs.copy_inputs:
            self.inputs.subjects_dir = os.getcwd()
            if "subjects_dir" in inputs:
                inputs["subjects_dir"] = self.inputs.subjects_dir
            hemi = self.inputs.hemisphere
            copy2subjdir(
                self, self.inputs.sphere_reg, "surf", "{0}.sphere.reg".format(hemi)
            )
            copy2subjdir(self, self.inputs.white, "surf", "{0}.white".format(hemi))
            copy2subjdir(
                self,
                self.inputs.source_sphere_reg,
                "surf",
                "{0}.sphere.reg".format(hemi),
                subject_id=self.inputs.source_subject,
            )
            copy2subjdir(
                self,
                self.inputs.source_white,
                "surf",
                "{0}.white".format(hemi),
                subject_id=self.inputs.source_subject,
            )

        # label dir must exist in order for output file to be written
        label_dir = os.path.join(
            self.inputs.subjects_dir, self.inputs.subject_id, "label"
        )
        if not os.path.isdir(label_dir):
            os.makedirs(label_dir)

        return super(Label2Label, self).run(**inputs)


class Label2AnnotInputSpec(FSTraitedSpec):
    # required
    hemisphere = traits.Enum(
        "lh", "rh", argstr="--hemi %s", mandatory=True, desc="Input hemisphere"
    )
    subject_id = traits.String(
        "subject_id",
        usedefault=True,
        argstr="--s %s",
        mandatory=True,
        desc="Subject name/ID",
    )
    in_labels = traits.List(
        argstr="--l %s...", mandatory=True, desc="List of input label files"
    )
    out_annot = traits.String(
        argstr="--a %s", mandatory=True, desc="Name of the annotation to create"
    )
    orig = File(exists=True, mandatory=True, desc="implicit {hemisphere}.orig")
    # optional
    keep_max = traits.Bool(
        argstr="--maxstatwinner", desc="Keep label with highest 'stat' value"
    )
    verbose_off = traits.Bool(
        argstr="--noverbose", desc="Turn off overlap and stat override messages"
    )
    color_table = File(
        argstr="--ctab %s",
        exists=True,
        desc="File that defines the structure names, their indices, and their color",
    )
    copy_inputs = traits.Bool(
        desc="copy implicit inputs and create a temp subjects_dir"
    )


class Label2AnnotOutputSpec(TraitedSpec):
    out_file = File(exists=True, desc="Output annotation file")


class Label2Annot(FSCommand):
    """
    Converts a set of surface labels to an annotation file

    Examples
    --------
    >>> from nipype.interfaces.freesurfer import Label2Annot
    >>> l2a = Label2Annot()
    >>> l2a.inputs.hemisphere = 'lh'
    >>> l2a.inputs.subject_id = '10335'
    >>> l2a.inputs.in_labels = ['lh.aparc.label']
    >>> l2a.inputs.orig = 'lh.pial'
    >>> l2a.inputs.out_annot = 'test'
    >>> l2a.cmdline
    'mris_label2annot --hemi lh --l lh.aparc.label --a test --s 10335'
    """

    _cmd = "mris_label2annot"
    input_spec = Label2AnnotInputSpec
    output_spec = Label2AnnotOutputSpec

    def run(self, **inputs):
        if self.inputs.copy_inputs:
            self.inputs.subjects_dir = os.getcwd()
            if "subjects_dir" in inputs:
                inputs["subjects_dir"] = self.inputs.subjects_dir
            copy2subjdir(
                self,
                self.inputs.orig,
                folder="surf",
                basename="{0}.orig".format(self.inputs.hemisphere),
            )
        # label dir must exist in order for output file to be written
        label_dir = os.path.join(
            self.inputs.subjects_dir, self.inputs.subject_id, "label"
        )
        if not os.path.isdir(label_dir):
            os.makedirs(label_dir)
        return super(Label2Annot, self).run(**inputs)

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs["out_file"] = os.path.join(
            str(self.inputs.subjects_dir),
            str(self.inputs.subject_id),
            "label",
            str(self.inputs.hemisphere) + "." + str(self.inputs.out_annot) + ".annot",
        )
        return outputs


class SphericalAverageInputSpec(FSTraitedSpec):
    out_file = File(
        argstr="%s", genfile=True, exists=False, position=-1, desc="Output filename"
    )
    in_average = Directory(
        argstr="%s", exists=True, genfile=True, position=-2, desc="Average subject"
    )
    in_surf = File(
        argstr="%s", mandatory=True, exists=True, position=-3, desc="Input surface file"
    )
    hemisphere = traits.Enum(
        "lh", "rh", argstr="%s", mandatory=True, position=-4, desc="Input hemisphere"
    )
    fname = traits.String(
        argstr="%s",
        mandatory=True,
        position=-5,
        desc="""\
Filename from the average subject directory.
Example: to use rh.entorhinal.label as the input label filename, set fname to 'rh.entorhinal'
and which to 'label'. The program will then search for
``<in_average>/label/rh.entorhinal.label``""",
    )
    which = traits.Enum(
        "coords",
        "label",
        "vals",
        "curv",
        "area",
        argstr="%s",
        mandatory=True,
        position=-6,
        desc="No documentation",
    )
    subject_id = traits.String(argstr="-o %s", mandatory=True, desc="Output subject id")
    # optional
    erode = traits.Int(argstr="-erode %d", desc="Undocumented")
    in_orig = File(argstr="-orig %s", exists=True, desc="Original surface filename")
    threshold = traits.Float(argstr="-t %.1f", desc="Undocumented")


class SphericalAverageOutputSpec(TraitedSpec):
    out_file = File(exists=False, desc="Output label")


class SphericalAverage(FSCommand):
    """
    This program will add a template into an average surface.

    Examples
    --------
    >>> from nipype.interfaces.freesurfer import SphericalAverage
    >>> sphericalavg = SphericalAverage()
    >>> sphericalavg.inputs.out_file = 'test.out'
    >>> sphericalavg.inputs.in_average = '.'
    >>> sphericalavg.inputs.in_surf = 'lh.pial'
    >>> sphericalavg.inputs.hemisphere = 'lh'
    >>> sphericalavg.inputs.fname = 'lh.entorhinal'
    >>> sphericalavg.inputs.which = 'label'
    >>> sphericalavg.inputs.subject_id = '10335'
    >>> sphericalavg.inputs.erode = 2
    >>> sphericalavg.inputs.threshold = 5
    >>> sphericalavg.cmdline
    'mris_spherical_average -erode 2 -o 10335 -t 5.0 label lh.entorhinal lh pial . test.out'

    """

    _cmd = "mris_spherical_average"
    input_spec = SphericalAverageInputSpec
    output_spec = SphericalAverageOutputSpec

    def _format_arg(self, name, spec, value):
        if name == "in_orig" or name == "in_surf":
            surf = os.path.basename(value)
            for item in ["lh.", "rh."]:
                surf = surf.replace(item, "")
            return spec.argstr % surf
        return super(SphericalAverage, self)._format_arg(name, spec, value)

    def _gen_filename(self, name):
        if name == "in_average":
            avg_subject = str(self.inputs.hemisphere) + ".EC_average"
            avg_directory = os.path.join(self.inputs.subjects_dir, avg_subject)
            if not os.path.isdir(avg_directory):
                fs_home = os.path.abspath(os.environ.get("FREESURFER_HOME"))
            return avg_subject
        elif name == "out_file":
            return self._list_outputs()[name]
        else:
            return None

    def _list_outputs(self):
        outputs = self._outputs().get()
        if isdefined(self.inputs.out_file):
            outputs["out_file"] = os.path.abspath(self.inputs.out_file)
        else:
            out_dir = os.path.join(
                self.inputs.subjects_dir, self.inputs.subject_id, "label"
            )
            if isdefined(self.inputs.in_average):
                basename = os.path.basename(self.inputs.in_average)
                basename = basename.replace("_", "_exvivo_") + ".label"
            else:
                basename = str(self.inputs.hemisphere) + ".EC_exvivo_average.label"
            outputs["out_file"] = os.path.join(out_dir, basename)
        return outputs



# -*- coding: utf-8 -*-
import os
from functools import lru_cache
import numpy as np
from numpy import ones, kron, mean, eye, hstack, tile
from numpy.linalg import pinv
import nibabel as nb
from ..interfaces.base import (
    BaseInterfaceInputSpec,
    TraitedSpec,
    BaseInterface,
    traits,
    File,
)


class ICCInputSpec(BaseInterfaceInputSpec):
    subjects_sessions = traits.List(
        traits.List(File(exists=True)),
        desc="n subjects m sessions 3D stat files",
        mandatory=True,
    )
    mask = File(exists=True, mandatory=True)


class ICCOutputSpec(TraitedSpec):
    icc_map = File(exists=True)
    session_var_map = File(exists=True, desc="variance between sessions")
    subject_var_map = File(exists=True, desc="variance between subjects")


class ICC(BaseInterface):
    """
    Calculates Interclass Correlation Coefficient (3,1) as defined in
    P. E. Shrout & Joseph L. Fleiss (1979). "Intraclass Correlations: Uses in
    Assessing Rater Reliability". Psychological Bulletin 86 (2): 420-428. This
    particular implementation is aimed at relaibility (test-retest) studies.
    """

    input_spec = ICCInputSpec
    output_spec = ICCOutputSpec

    def _run_interface(self, runtime):
        maskdata = nb.load(self.inputs.mask).get_fdata()
        maskdata = np.logical_not(np.logical_or(maskdata == 0, np.isnan(maskdata)))

        session_datas = [
            [nb.load(fname).get_fdata()[maskdata].reshape(-1, 1) for fname in sessions]
            for sessions in self.inputs.subjects_sessions
        ]
        list_of_sessions = [np.dstack(session_data) for session_data in session_datas]
        all_data = np.hstack(list_of_sessions)
        icc = np.zeros(session_datas[0][0].shape)
        session_F = np.zeros(session_datas[0][0].shape)
        session_var = np.zeros(session_datas[0][0].shape)
        subject_var = np.zeros(session_datas[0][0].shape)

        for x in range(icc.shape[0]):
            Y = all_data[x, :, :]
            icc[x], subject_var[x], session_var[x], session_F[x], _, _ = ICC_rep_anova(
                Y
            )

        nim = nb.load(self.inputs.subjects_sessions[0][0])
        new_data = np.zeros(nim.shape)
        new_data[maskdata] = icc.reshape(-1)
        new_img = nb.Nifti1Image(new_data, nim.affine, nim.header)
        nb.save(new_img, "icc_map.nii")

        new_data = np.zeros(nim.shape)
        new_data[maskdata] = session_var.reshape(-1)
        new_img = nb.Nifti1Image(new_data, nim.affine, nim.header)
        nb.save(new_img, "session_var_map.nii")

        new_data = np.zeros(nim.shape)
        new_data[maskdata] = subject_var.reshape(-1)
        new_img = nb.Nifti1Image(new_data, nim.affine, nim.header)
        nb.save(new_img, "subject_var_map.nii")

        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs["icc_map"] = os.path.abspath("icc_map.nii")
        outputs["session_var_map"] = os.path.abspath("session_var_map.nii")
        outputs["subject_var_map"] = os.path.abspath("subject_var_map.nii")
        return outputs


@lru_cache(maxsize=1)
def ICC_projection_matrix(shape):
    nb_subjects, nb_conditions = shape

    x = kron(eye(nb_conditions), ones((nb_subjects, 1)))  # sessions
    x0 = tile(eye(nb_subjects), (nb_conditions, 1))  # subjects
    X = hstack([x, x0])
    return X @ pinv(X.T @ X, hermitian=True) @ X.T


def ICC_rep_anova(Y, projection_matrix=None):
    """
    the data Y are entered as a 'table' ie subjects are in rows and repeated
    measures in columns

    One Sample Repeated measure ANOVA

    Y = XB + E with X = [FaTor / Subjects]

    ``ICC_rep_anova`` involves an expensive operation to compute a projection
    matrix, which depends only on the shape of ``Y``, which is computed by
    calling ``ICC_projection_matrix(Y.shape)``. If arrays of multiple shapes are
    expected, it may be worth pre-computing and passing directly as an
    argument to ``ICC_rep_anova``.

    If only one ``Y.shape`` will occur, you do not need to explicitly handle
    these, as the most recently calculated matrix is cached automatically.
    For example, if you are running the same computation on every voxel of
    an image, you will see significant speedups.

    If a ``Y`` is passed with a new shape, a new matrix will be calculated
    automatically.
    """
    [nb_subjects, nb_conditions] = Y.shape
    dfc = nb_conditions - 1
    dfr = nb_subjects - 1
    dfe = dfr * dfc

    # Compute the repeated measure effect
    # ------------------------------------

    # Sum Square Total
    demeaned_Y = Y - mean(Y)
    SST = np.sum(demeaned_Y**2)

    # Sum Square Error
    if projection_matrix is None:
        projection_matrix = ICC_projection_matrix(Y.shape)
    residuals = Y.flatten("F") - (projection_matrix @ Y.flatten("F"))
    SSE = np.sum(residuals**2)

    MSE = SSE / dfe

    # Sum square session effect - between columns/sessions
    SSC = np.sum(mean(demeaned_Y, 0) ** 2) * nb_subjects
    MSC = SSC / dfc / nb_subjects

    session_effect_F = MSC / MSE

    # Sum Square subject effect - between rows/subjects
    SSR = SST - SSC - SSE
    MSR = SSR / dfr

    # ICC(3,1) = (mean square subjeT - mean square error) /
    #            (mean square subjeT + (k-1)*-mean square error)
    ICC = (MSR - MSE) / (MSR + dfc * MSE)

    e_var = MSE  # variance of error
    r_var = (MSR - MSE) / nb_conditions  # variance between subjects

    return ICC, r_var, e_var, session_effect_F, dfc, dfe



# -*- coding: utf-8 -*-
import os
import nipype.interfaces.base as nib

import nipype.pipeline.engine as pe


class InputSpec(nib.TraitedSpec):
    input1 = nib.traits.Int(desc="a random int")
    input2 = nib.traits.Int(desc="a random int")


class OutputSpec(nib.TraitedSpec):
    output1 = nib.traits.List(nib.traits.Int, desc="outputs")


class LinearTestInterface(nib.BaseInterface):
    input_spec = InputSpec
    output_spec = OutputSpec

    def _run_interface(self, runtime):
        runtime.returncode = 0
        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs["output1"] = [1, self.inputs.input1]
        return outputs


def test_run_in_series(tmpdir):
    tmpdir.chdir()

    pipe = pe.Workflow(name="pipe")
    mod1 = pe.Node(interface=LinearTestInterface(), name="mod1")
    mod2 = pe.MapNode(
        interface=LinearTestInterface(), iterfield=["input1"], name="mod2"
    )
    pipe.connect([(mod1, mod2, [("output1", "input1")])])
    pipe.base_dir = os.getcwd()
    mod1.inputs.input1 = 1
    execgraph = pipe.run(plugin="Linear")
    names = [".".join((node._hierarchy, node.name)) for node in execgraph.nodes()]
    node = list(execgraph.nodes())[names.index("pipe.mod1")]
    result = node.get_output("output1")
    assert result == [1, 1]



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..model import MRISPreprocReconAll


def test_MRISPreprocReconAll_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        copy_inputs=dict(),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fsgd_file=dict(
            argstr="--fsgd %s",
            extensions=None,
            xor=("subjects", "fsgd_file", "subject_file"),
        ),
        fwhm=dict(
            argstr="--fwhm %f",
            xor=["num_iters"],
        ),
        fwhm_source=dict(
            argstr="--fwhm-src %f",
            xor=["num_iters_source"],
        ),
        hemi=dict(
            argstr="--hemi %s",
            mandatory=True,
        ),
        lh_surfreg_target=dict(
            extensions=None,
            requires=["surfreg_files"],
        ),
        num_iters=dict(
            argstr="--niters %d",
            xor=["fwhm"],
        ),
        num_iters_source=dict(
            argstr="--niterssrc %d",
            xor=["fwhm_source"],
        ),
        out_file=dict(
            argstr="--out %s",
            extensions=None,
            genfile=True,
        ),
        proj_frac=dict(
            argstr="--projfrac %s",
        ),
        rh_surfreg_target=dict(
            extensions=None,
            requires=["surfreg_files"],
        ),
        smooth_cortex_only=dict(
            argstr="--smooth-cortex-only",
        ),
        source_format=dict(
            argstr="--srcfmt %s",
        ),
        subject_file=dict(
            argstr="--f %s",
            extensions=None,
            xor=("subjects", "fsgd_file", "subject_file"),
        ),
        subject_id=dict(
            argstr="--s %s",
            usedefault=True,
            xor=("subjects", "fsgd_file", "subject_file", "subject_id"),
        ),
        subjects=dict(
            argstr="--s %s...",
            xor=("subjects", "fsgd_file", "subject_file"),
        ),
        subjects_dir=dict(),
        surf_area=dict(
            argstr="--area %s",
            xor=("surf_measure", "surf_measure_file", "surf_area"),
        ),
        surf_dir=dict(
            argstr="--surfdir %s",
        ),
        surf_measure=dict(
            argstr="--meas %s",
            xor=("surf_measure", "surf_measure_file", "surf_area"),
        ),
        surf_measure_file=dict(
            argstr="--meas %s",
            extensions=None,
            xor=("surf_measure", "surf_measure_file", "surf_area"),
        ),
        surfreg_files=dict(
            argstr="--surfreg %s",
            requires=["lh_surfreg_target", "rh_surfreg_target"],
        ),
        target=dict(
            argstr="--target %s",
            mandatory=True,
        ),
        vol_measure_file=dict(
            argstr="--iv %s %s...",
        ),
    )
    inputs = MRISPreprocReconAll.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_MRISPreprocReconAll_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = MRISPreprocReconAll.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..connectivity import Conmat


def test_Conmat_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr="-inputfile %s",
            extensions=None,
            mandatory=True,
        ),
        output_root=dict(
            argstr="-outputroot %s",
            extensions=None,
            genfile=True,
        ),
        scalar_file=dict(
            argstr="-scalarfile %s",
            extensions=None,
            requires=["tract_stat"],
        ),
        target_file=dict(
            argstr="-targetfile %s",
            extensions=None,
            mandatory=True,
        ),
        targetname_file=dict(
            argstr="-targetnamefile %s",
            extensions=None,
        ),
        tract_prop=dict(
            argstr="-tractstat %s",
            units="NA",
            xor=["tract_stat"],
        ),
        tract_stat=dict(
            argstr="-tractstat %s",
            requires=["scalar_file"],
            units="NA",
            xor=["tract_prop"],
        ),
    )
    inputs = Conmat.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Conmat_outputs():
    output_map = dict(
        conmat_sc=dict(
            extensions=None,
        ),
        conmat_ts=dict(
            extensions=None,
        ),
    )
    outputs = Conmat.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

from ..segmentation import LaplacianThickness
from .test_resampling import change_dir

import os
import pytest


@pytest.fixture()
def change_dir(request):
    orig_dir = os.getcwd()
    filepath = os.path.dirname(os.path.realpath(__file__))
    datadir = os.path.realpath(os.path.join(filepath, "../../../testing/data"))
    os.chdir(datadir)

    def move2orig():
        os.chdir(orig_dir)

    request.addfinalizer(move2orig)


@pytest.fixture()
def create_lt():
    lt = LaplacianThickness()
    # we do not run, so I stick some not really proper files as input
    lt.inputs.input_gm = "diffusion_weighted.nii"
    lt.inputs.input_wm = "functional.nii"
    return lt


def test_LaplacianThickness_defaults(change_dir, create_lt):
    lt = create_lt
    base_cmd = "LaplacianThickness functional.nii diffusion_weighted.nii functional_thickness.nii"
    assert lt.cmdline == base_cmd
    lt.inputs.smooth_param = 4.5
    assert lt.cmdline == base_cmd + " 4.5"
    lt.inputs.prior_thickness = 5.9
    assert lt.cmdline == base_cmd + " 4.5 5.9"


def test_LaplacianThickness_wrongargs(change_dir, create_lt):
    lt = create_lt
    lt.inputs.tolerance = 0.001
    with pytest.raises(
        ValueError, match=r".* requires a value for input 'sulcus_prior' .*"
    ):
        lt.cmdline
    lt.inputs.sulcus_prior = 0.15
    with pytest.raises(ValueError, match=r".* requires a value for input 'dT' .*"):
        lt.cmdline
    lt.inputs.dT = 0.01
    with pytest.raises(
        ValueError, match=r".* requires a value for input 'prior_thickness' .*"
    ):
        lt.cmdline
    lt.inputs.prior_thickness = 5.9
    with pytest.raises(
        ValueError, match=r".* requires a value for input 'smooth_param' .*"
    ):
        lt.cmdline
    lt.inputs.smooth_param = 4.5
    assert (
        lt.cmdline == "LaplacianThickness functional.nii diffusion_weighted.nii "
        "functional_thickness.nii 4.5 5.9 0.01 0.15 0.001"
    )



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..model import Level1Design


def test_Level1Design_inputs():
    input_map = dict(
        bases=dict(
            field="bases",
            mandatory=True,
        ),
        factor_info=dict(
            field="fact",
        ),
        flags=dict(),
        global_intensity_normalization=dict(
            field="global",
        ),
        interscan_interval=dict(
            field="timing.RT",
            mandatory=True,
        ),
        mask_image=dict(
            extensions=None,
            field="mask",
        ),
        mask_threshold=dict(
            usedefault=True,
        ),
        matlab_cmd=dict(),
        mfile=dict(
            usedefault=True,
        ),
        microtime_onset=dict(
            field="timing.fmri_t0",
        ),
        microtime_resolution=dict(
            field="timing.fmri_t",
        ),
        model_serial_correlations=dict(
            field="cvi",
        ),
        paths=dict(),
        session_info=dict(
            field="sess",
            mandatory=True,
        ),
        spm_mat_dir=dict(
            field="dir",
        ),
        timing_units=dict(
            field="timing.units",
            mandatory=True,
        ),
        use_mcr=dict(),
        use_v8struct=dict(
            min_ver="8",
            usedefault=True,
        ),
        volterra_expansion_order=dict(
            field="volt",
        ),
    )
    inputs = Level1Design.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Level1Design_outputs():
    output_map = dict(
        spm_mat_file=dict(
            extensions=None,
        ),
    )
    outputs = Level1Design.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import Unifize


def test_Unifize_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        cl_frac=dict(
            argstr="-clfrac %f",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        epi=dict(
            argstr="-EPI",
            requires=["no_duplo", "t2"],
            xor=["gm"],
        ),
        gm=dict(
            argstr="-GM",
        ),
        in_file=dict(
            argstr="-input %s",
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        no_duplo=dict(
            argstr="-noduplo",
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_file=dict(
            argstr="-prefix %s",
            extensions=None,
            name_source="in_file",
            name_template="%s_unifized",
        ),
        outputtype=dict(),
        quiet=dict(
            argstr="-quiet",
        ),
        rbt=dict(
            argstr="-rbt %f %f %f",
        ),
        scale_file=dict(
            argstr="-ssave %s",
            extensions=None,
        ),
        t2=dict(
            argstr="-T2",
        ),
        t2_up=dict(
            argstr="-T2up %f",
        ),
        urad=dict(
            argstr="-Urad %s",
        ),
    )
    inputs = Unifize.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Unifize_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
        scale_file=dict(
            extensions=None,
        ),
    )
    outputs = Unifize.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..simpleregiongrowingsegmentation import SimpleRegionGrowingSegmentation


def test_SimpleRegionGrowingSegmentation_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputVolume=dict(
            argstr="%s",
            extensions=None,
            position=-2,
        ),
        iterations=dict(
            argstr="--iterations %d",
        ),
        labelvalue=dict(
            argstr="--labelvalue %d",
        ),
        multiplier=dict(
            argstr="--multiplier %f",
        ),
        neighborhood=dict(
            argstr="--neighborhood %d",
        ),
        outputVolume=dict(
            argstr="%s",
            hash_files=False,
            position=-1,
        ),
        seed=dict(
            argstr="--seed %s...",
        ),
        smoothingIterations=dict(
            argstr="--smoothingIterations %d",
        ),
        timestep=dict(
            argstr="--timestep %f",
        ),
    )
    inputs = SimpleRegionGrowingSegmentation.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_SimpleRegionGrowingSegmentation_outputs():
    output_map = dict(
        outputVolume=dict(
            extensions=None,
            position=-1,
        ),
    )
    outputs = SimpleRegionGrowingSegmentation.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..resampling import ApplyTransforms


def test_ApplyTransforms_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        default_value=dict(
            argstr="--default-value %g",
            usedefault=True,
        ),
        dimension=dict(
            argstr="--dimensionality %d",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        float=dict(
            argstr="--float %d",
            usedefault=True,
        ),
        input_image=dict(
            argstr="--input %s",
            extensions=None,
            mandatory=True,
        ),
        input_image_type=dict(
            argstr="--input-image-type %d",
        ),
        interpolation=dict(
            argstr="%s",
            usedefault=True,
        ),
        interpolation_parameters=dict(),
        invert_transform_flags=dict(),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_postfix=dict(
            usedefault=True,
        ),
        output_image=dict(
            argstr="--output %s",
            genfile=True,
            hash_files=False,
        ),
        print_out_composite_warp_file=dict(
            requires=["output_image"],
        ),
        reference_image=dict(
            argstr="--reference-image %s",
            extensions=None,
            mandatory=True,
        ),
        transforms=dict(
            argstr="%s",
            mandatory=True,
        ),
    )
    inputs = ApplyTransforms.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_ApplyTransforms_outputs():
    output_map = dict(
        output_image=dict(
            extensions=None,
        ),
    )
    outputs = ApplyTransforms.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

from copy import deepcopy
import os

from nibabel import Nifti1Image
import numpy as np

import pytest
import numpy.testing as npt
from nipype.testing import example_data
from nipype.interfaces.base import Bunch, TraitError
from nipype.algorithms.modelgen import (
    bids_gen_info,
    SpecifyModel,
    SpecifySparseModel,
    SpecifySPMModel,
)


def test_bids_gen_info():
    fname = example_data("events.tsv")
    res = bids_gen_info([fname])
    assert res[0].onsets == [
        [183.75, 313.75, 483.75, 633.75, 783.75, 933.75, 1083.75, 1233.75]
    ]
    assert res[0].durations == [[20.0, 20.0, 20.0, 20.0, 20.0, 20.0, 20.0, 20.0]]
    assert res[0].amplitudes == [[1, 1, 1, 1, 1, 1, 1, 1]]
    assert res[0].conditions == ["ev0"]


def test_modelgen1(tmpdir):
    filename1 = tmpdir.join("test1.nii").strpath
    filename2 = tmpdir.join("test2.nii").strpath
    Nifti1Image(np.random.rand(10, 10, 10, 200), np.eye(4)).to_filename(filename1)
    Nifti1Image(np.random.rand(10, 10, 10, 200), np.eye(4)).to_filename(filename2)
    s = SpecifyModel()
    s.inputs.input_units = "scans"
    set_output_units = lambda: setattr(s.inputs, "output_units", "scans")
    with pytest.raises(TraitError):
        set_output_units()
    s.inputs.functional_runs = [filename1, filename2]
    s.inputs.time_repetition = 6
    s.inputs.high_pass_filter_cutoff = 128.0
    info = [
        Bunch(
            conditions=["cond1"],
            onsets=[[2, 50, 100, 180]],
            durations=[[1]],
            amplitudes=None,
            pmod=None,
            regressors=None,
            regressor_names=None,
            tmod=None,
        ),
        Bunch(
            conditions=["cond1"],
            onsets=[[30, 40, 100, 150]],
            durations=[[1]],
            amplitudes=None,
            pmod=None,
            regressors=None,
            regressor_names=None,
            tmod=None,
        ),
    ]
    s.inputs.subject_info = info
    res = s.run()
    assert len(res.outputs.session_info) == 2
    assert len(res.outputs.session_info[0]["regress"]) == 0
    assert len(res.outputs.session_info[0]["cond"]) == 1
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]["cond"][0]["onset"]),
        np.array([12, 300, 600, 1080]),
    )
    info = [
        Bunch(conditions=["cond1"], onsets=[[2]], durations=[[1]]),
        Bunch(conditions=["cond1"], onsets=[[3]], durations=[[1]]),
    ]
    s.inputs.subject_info = deepcopy(info)
    res = s.run()
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]["cond"][0]["duration"]), np.array([6.0])
    )
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[1]["cond"][0]["duration"]), np.array([6.0])
    )
    info = [
        Bunch(
            conditions=["cond1", "cond2"], onsets=[[2, 3], [2]], durations=[[1, 1], [1]]
        ),
        Bunch(
            conditions=["cond1", "cond2"],
            onsets=[[2, 3], [2, 4]],
            durations=[[1, 1], [1, 1]],
        ),
    ]
    s.inputs.subject_info = deepcopy(info)
    s.inputs.input_units = "scans"
    res = s.run()
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]["cond"][0]["duration"]),
        np.array([6.0, 6.0]),
    )
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]["cond"][1]["duration"]), np.array([6.0])
    )
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[1]["cond"][1]["duration"]),
        np.array([6.0, 6.0]),
    )


def test_modelgen_spm_concat(tmpdir):
    filename1 = tmpdir.join("test1.nii").strpath
    filename2 = tmpdir.join("test2.nii").strpath
    Nifti1Image(np.random.rand(10, 10, 10, 30), np.eye(4)).to_filename(filename1)
    Nifti1Image(np.random.rand(10, 10, 10, 30), np.eye(4)).to_filename(filename2)

    # Test case when only one duration is passed, as being the same for all onsets.
    s = SpecifySPMModel()
    s.inputs.input_units = "secs"
    s.inputs.concatenate_runs = True
    setattr(s.inputs, "output_units", "secs")
    assert s.inputs.output_units == "secs"
    s.inputs.functional_runs = [filename1, filename2]
    s.inputs.time_repetition = 6
    s.inputs.high_pass_filter_cutoff = 128.0
    info = [
        Bunch(conditions=["cond1"], onsets=[[2, 50, 100, 170]], durations=[[1]]),
        Bunch(conditions=["cond1"], onsets=[[30, 40, 100, 150]], durations=[[1]]),
    ]
    s.inputs.subject_info = deepcopy(info)
    res = s.run()
    assert len(res.outputs.session_info) == 1
    assert len(res.outputs.session_info[0]["regress"]) == 1
    assert np.sum(res.outputs.session_info[0]["regress"][0]["val"]) == 30
    assert len(res.outputs.session_info[0]["cond"]) == 1
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]["cond"][0]["onset"]),
        np.array([2.0, 50.0, 100.0, 170.0, 210.0, 220.0, 280.0, 330.0]),
    )
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]["cond"][0]["duration"]),
        np.array([1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0]),
    )

    # Test case of scans as output units instead of seconds
    setattr(s.inputs, "output_units", "scans")
    assert s.inputs.output_units == "scans"
    s.inputs.subject_info = deepcopy(info)
    res = s.run()
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]["cond"][0]["onset"]),
        np.array([2.0, 50.0, 100.0, 170.0, 210.0, 220.0, 280.0, 330.0]) / 6,
    )

    # Test case for no concatenation with seconds as output units
    s.inputs.concatenate_runs = False
    s.inputs.subject_info = deepcopy(info)
    s.inputs.output_units = "secs"
    res = s.run()
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]["cond"][0]["onset"]),
        np.array([2.0, 50.0, 100.0, 170.0]),
    )

    # Test case for variable number of events in separate runs, sometimes unique.
    filename3 = tmpdir.join("test3.nii").strpath
    Nifti1Image(np.random.rand(10, 10, 10, 30), np.eye(4)).to_filename(filename3)
    s.inputs.functional_runs = [filename1, filename2, filename3]
    info = [
        Bunch(
            conditions=["cond1", "cond2"], onsets=[[2, 3], [2]], durations=[[1, 1], [1]]
        ),
        Bunch(
            conditions=["cond1", "cond2"],
            onsets=[[2, 3], [2, 4]],
            durations=[[1, 1], [1, 1]],
        ),
        Bunch(
            conditions=["cond1", "cond2"], onsets=[[2, 3], [2]], durations=[[1, 1], [1]]
        ),
    ]
    s.inputs.subject_info = deepcopy(info)
    res = s.run()
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]["cond"][0]["duration"]),
        np.array([1.0, 1.0]),
    )
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]["cond"][1]["duration"]), np.array([1.0])
    )
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[1]["cond"][1]["duration"]),
        np.array([1.0, 1.0]),
    )
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[2]["cond"][1]["duration"]), np.array([1.0])
    )

    # Test case for variable number of events in concatenated runs, sometimes unique.
    s.inputs.concatenate_runs = True
    info = [
        Bunch(
            conditions=["cond1", "cond2"], onsets=[[2, 3], [2]], durations=[[1, 1], [1]]
        ),
        Bunch(
            conditions=["cond1", "cond2"],
            onsets=[[2, 3], [2, 4]],
            durations=[[1, 1], [1, 1]],
        ),
        Bunch(
            conditions=["cond1", "cond2"], onsets=[[2, 3], [2]], durations=[[1, 1], [1]]
        ),
    ]
    s.inputs.subject_info = deepcopy(info)
    res = s.run()
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]["cond"][0]["duration"]),
        np.array([1.0, 1.0, 1.0, 1.0, 1.0, 1.0]),
    )
    npt.assert_almost_equal(
        np.array(res.outputs.session_info[0]["cond"][1]["duration"]),
        np.array([1.0, 1.0, 1.0, 1.0]),
    )


def test_modelgen_sparse(tmpdir):
    filename1 = tmpdir.join("test1.nii").strpath
    filename2 = tmpdir.join("test2.nii").strpath
    Nifti1Image(np.random.rand(10, 10, 10, 50), np.eye(4)).to_filename(filename1)
    Nifti1Image(np.random.rand(10, 10, 10, 50), np.eye(4)).to_filename(filename2)
    s = SpecifySparseModel()
    s.inputs.input_units = "secs"
    s.inputs.functional_runs = [filename1, filename2]
    s.inputs.time_repetition = 6
    info = [
        Bunch(conditions=["cond1"], onsets=[[0, 50, 100, 180]], durations=[[2]]),
        Bunch(conditions=["cond1"], onsets=[[30, 40, 100, 150]], durations=[[1]]),
    ]
    s.inputs.subject_info = info
    s.inputs.volumes_in_cluster = 1
    s.inputs.time_acquisition = 2
    s.inputs.high_pass_filter_cutoff = np.inf
    res = s.run()
    assert len(res.outputs.session_info) == 2
    assert len(res.outputs.session_info[0]["regress"]) == 1
    assert len(res.outputs.session_info[0]["cond"]) == 0

    s.inputs.stimuli_as_impulses = False
    res = s.run()
    assert res.outputs.session_info[0]["regress"][0]["val"][0] == 1.0

    s.inputs.model_hrf = True
    res = s.run()
    npt.assert_almost_equal(
        res.outputs.session_info[0]["regress"][0]["val"][0], 0.016675298129743384
    )
    assert len(res.outputs.session_info[0]["regress"]) == 1
    s.inputs.use_temporal_deriv = True
    res = s.run()

    assert len(res.outputs.session_info[0]["regress"]) == 2
    npt.assert_almost_equal(
        res.outputs.session_info[0]["regress"][0]["val"][0], 0.016675298129743384
    )
    npt.assert_almost_equal(
        res.outputs.session_info[1]["regress"][1]["val"][5], 0.007671459162258378
    )



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Top-level namespace for ants."""

# Registration programs
from .registration import (
    ANTS,
    CompositeTransformUtil,
    MeasureImageSimilarity,
    Registration,
    RegistrationSynQuick,
)

# Resampling Programs
from .resampling import (
    ApplyTransforms,
    ApplyTransformsToPoints,
    WarpImageMultiTransform,
    WarpTimeSeriesImageMultiTransform,
)

# Segmentation Programs
from .segmentation import (
    AntsJointFusion,
    Atropos,
    BrainExtraction,
    CorticalThickness,
    DenoiseImage,
    JointFusion,
    LaplacianThickness,
    N4BiasFieldCorrection,
)

# Visualization Programs
from .visualization import ConvertScalarImageToRGB, CreateTiledMosaic

# Utility Programs
from .utils import (
    AffineInitializer,
    AI,
    AverageAffineTransform,
    AverageImages,
    ComposeMultiTransform,
    CreateJacobianDeterminantImage,
    ImageMath,
    LabelGeometry,
    MultiplyImages,
    ResampleImageBySpacing,
    ThresholdImage,
)

__all__ = [
    "AffineInitializer",
    "AI",
    "ANTS",
    "AntsJointFusion",
    "ApplyTransforms",
    "ApplyTransformsToPoints",
    "Atropos",
    "AverageAffineTransform",
    "AverageImages",
    "BrainExtraction",
    "ComposeMultiTransform",
    "CompositeTransformUtil",
    "ConvertScalarImageToRGB",
    "CorticalThickness",
    "CreateJacobianDeterminantImage",
    "CreateTiledMosaic",
    "DenoiseImage",
    "ImageMath",
    "JointFusion",
    "LabelGeometry",
    "LaplacianThickness",
    "MeasureImageSimilarity",
    "MultiplyImages",
    "N4BiasFieldCorrection",
    "Registration",
    "RegistrationSynQuick",
    "ResampleImageBySpacing",
    "ThresholdImage",
    "WarpImageMultiTransform",
    "WarpTimeSeriesImageMultiTransform",
]



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

from .debug import DebugPlugin
from .linear import LinearPlugin
from .pbs import PBSPlugin
from .oar import OARPlugin
from .sge import SGEPlugin
from .condor import CondorPlugin
from .dagman import CondorDAGManPlugin
from .multiproc import MultiProcPlugin
from .legacymultiproc import LegacyMultiProcPlugin
from .ipython import IPythonPlugin
from .somaflow import SomaFlowPlugin
from .pbsgraph import PBSGraphPlugin
from .sgegraph import SGEGraphPlugin
from .lsf import LSFPlugin
from .slurm import SLURMPlugin
from .slurmgraph import SLURMGraphPlugin

from . import semaphore_singleton



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..preprocess import Volreg


def test_Volreg_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        basefile=dict(
            argstr="-base %s",
            extensions=None,
            position=-6,
        ),
        copyorigin=dict(
            argstr="-twodup",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr="%s",
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        in_weight_volume=dict(
            argstr="-weight '%s[%d]'",
        ),
        interp=dict(
            argstr="-%s",
        ),
        md1d_file=dict(
            argstr="-maxdisp1D %s",
            extensions=None,
            keep_extension=True,
            name_source="in_file",
            name_template="%s_md.1D",
            position=-4,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        oned_file=dict(
            argstr="-1Dfile %s",
            extensions=None,
            keep_extension=True,
            name_source="in_file",
            name_template="%s.1D",
        ),
        oned_matrix_save=dict(
            argstr="-1Dmatrix_save %s",
            extensions=None,
            keep_extension=True,
            name_source="in_file",
            name_template="%s.aff12.1D",
        ),
        out_file=dict(
            argstr="-prefix %s",
            extensions=None,
            name_source="in_file",
            name_template="%s_volreg",
        ),
        outputtype=dict(),
        timeshift=dict(
            argstr="-tshift 0",
        ),
        verbose=dict(
            argstr="-verbose",
        ),
        zpad=dict(
            argstr="-zpad %d",
            position=-5,
        ),
    )
    inputs = Volreg.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Volreg_outputs():
    output_map = dict(
        md1d_file=dict(
            extensions=None,
        ),
        oned_file=dict(
            extensions=None,
        ),
        oned_matrix_save=dict(
            extensions=None,
        ),
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = Volreg.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..dti import DTIRecon


def test_DTIRecon_inputs():
    input_map = dict(
        DWI=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=1,
        ),
        args=dict(
            argstr="%s",
        ),
        b0_threshold=dict(
            argstr="-b0_th",
        ),
        bvals=dict(
            extensions=None,
            mandatory=True,
        ),
        bvecs=dict(
            argstr="-gm %s",
            extensions=None,
            mandatory=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        image_orientation_vectors=dict(
            argstr="-iop %f",
        ),
        n_averages=dict(
            argstr="-nex %s",
        ),
        oblique_correction=dict(
            argstr="-oc",
        ),
        out_prefix=dict(
            argstr="%s",
            position=2,
            usedefault=True,
        ),
        output_type=dict(
            argstr="-ot %s",
            usedefault=True,
        ),
    )
    inputs = DTIRecon.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_DTIRecon_outputs():
    output_map = dict(
        ADC=dict(
            extensions=None,
        ),
        B0=dict(
            extensions=None,
        ),
        FA=dict(
            extensions=None,
        ),
        FA_color=dict(
            extensions=None,
        ),
        L1=dict(
            extensions=None,
        ),
        L2=dict(
            extensions=None,
        ),
        L3=dict(
            extensions=None,
        ),
        V1=dict(
            extensions=None,
        ),
        V2=dict(
            extensions=None,
        ),
        V3=dict(
            extensions=None,
        ),
        exp=dict(
            extensions=None,
        ),
        tensor=dict(
            extensions=None,
        ),
    )
    outputs = DTIRecon.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
"""Parallel workflow execution via LSF
"""

import os
import re
from time import sleep

from ... import logging
from ...interfaces.base import CommandLine
from .base import SGELikeBatchManagerBase, logger

iflogger = logging.getLogger("nipype.interface")


class LSFPlugin(SGELikeBatchManagerBase):
    """Execute using LSF Cluster Submission

    The plugin_args input to run can be used to control the LSF execution.
    Currently supported options are:

    - template : template to use for batch job submission
    - bsub_args : arguments to be prepended to the job execution script in the
                  bsub call

    """

    def __init__(self, **kwargs):
        template = """
#$ -S /bin/sh
        """
        self._retry_timeout = 2
        self._max_tries = 2
        self._bsub_args = ""
        if "plugin_args" in kwargs and kwargs["plugin_args"]:
            if "retry_timeout" in kwargs["plugin_args"]:
                self._retry_timeout = kwargs["plugin_args"]["retry_timeout"]
            if "max_tries" in kwargs["plugin_args"]:
                self._max_tries = kwargs["plugin_args"]["max_tries"]
            if "bsub_args" in kwargs["plugin_args"]:
                self._bsub_args = kwargs["plugin_args"]["bsub_args"]
        super(LSFPlugin, self).__init__(template, **kwargs)

    def _is_pending(self, taskid):
        """LSF lists a status of 'PEND' when a job has been submitted but is
        waiting to be picked up, and 'RUN' when it is actively being processed.
        But _is_pending should return True until a job has finished and is
        ready to be checked for completeness. So return True if status is
        either 'PEND' or 'RUN'"""
        cmd = CommandLine("bjobs", resource_monitor=False, terminal_output="allatonce")
        cmd.inputs.args = "%d" % taskid
        # check lsf task
        oldlevel = iflogger.level
        iflogger.setLevel(logging.getLevelName("CRITICAL"))
        result = cmd.run(ignore_exception=True)
        iflogger.setLevel(oldlevel)
        # logger.debug(result.runtime.stdout)
        if "DONE" in result.runtime.stdout or "EXIT" in result.runtime.stdout:
            return False
        else:
            return True

    def _submit_batchtask(self, scriptfile, node):
        cmd = CommandLine(
            "bsub",
            environ=dict(os.environ),
            resource_monitor=False,
            terminal_output="allatonce",
        )
        bsubargs = ""
        if self._bsub_args:
            bsubargs = self._bsub_args
        if "bsub_args" in node.plugin_args:
            if "overwrite" in node.plugin_args and node.plugin_args["overwrite"]:
                bsubargs = node.plugin_args["bsub_args"]
            else:
                bsubargs += " " + node.plugin_args["bsub_args"]
        if "-o" not in bsubargs:  # -o outfile
            bsubargs = "%s -o %s" % (bsubargs, scriptfile + ".log")
        if "-e" not in bsubargs:
            # -e error file
            bsubargs = "%s -e %s" % (bsubargs, scriptfile + ".log")
        if node._hierarchy:
            jobname = ".".join((dict(os.environ)["LOGNAME"], node._hierarchy, node._id))
        else:
            jobname = ".".join((dict(os.environ)["LOGNAME"], node._id))
        jobnameitems = jobname.split(".")
        jobnameitems.reverse()
        jobname = ".".join(jobnameitems)
        cmd.inputs.args = "%s -J %s sh %s" % (
            bsubargs,
            jobname,
            scriptfile,
        )  # -J job_name_spec
        logger.debug("bsub " + cmd.inputs.args)
        oldlevel = iflogger.level
        iflogger.setLevel(logging.getLevelName("CRITICAL"))
        tries = 0
        while True:
            try:
                result = cmd.run()
            except Exception as e:
                if tries < self._max_tries:
                    tries += 1
                    sleep(self._retry_timeout)  # sleep 2 seconds and try again.
                else:
                    iflogger.setLevel(oldlevel)
                    raise RuntimeError(
                        "\n".join(
                            (
                                ("Could not submit lsf task" " for node %s") % node._id,
                                str(e),
                            )
                        )
                    )
            else:
                break
        iflogger.setLevel(oldlevel)
        # retrieve lsf taskid
        match = re.search(r"<(\d*)>", result.runtime.stdout)
        if match:
            taskid = int(match.groups()[0])
        else:
            raise IOError(
                "Can't parse submission job output id: %s" % result.runtime.stdout
            )
        self._pending[taskid] = node.output_dir()
        logger.debug("submitted lsf task: %d for node %s" % (taskid, node._id))
        return taskid



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..epi import Eddy


def test_Eddy_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        cnr_maps=dict(
            argstr="--cnr_maps",
            min_ver="5.0.10",
        ),
        dont_peas=dict(
            argstr="--dont_peas",
        ),
        dont_sep_offs_move=dict(
            argstr="--dont_sep_offs_move",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        estimate_move_by_susceptibility=dict(
            argstr="--estimate_move_by_susceptibility",
            min_ver="6.0.1",
        ),
        fep=dict(
            argstr="--fep",
        ),
        field=dict(
            argstr="--field=%s",
            extensions=None,
        ),
        field_mat=dict(
            argstr="--field_mat=%s",
            extensions=None,
        ),
        flm=dict(
            argstr="--flm=%s",
            usedefault=True,
        ),
        fudge_factor=dict(
            argstr="--ff=%s",
            usedefault=True,
        ),
        fwhm=dict(
            argstr="--fwhm=%s",
        ),
        in_acqp=dict(
            argstr="--acqp=%s",
            extensions=None,
            mandatory=True,
        ),
        in_bval=dict(
            argstr="--bvals=%s",
            extensions=None,
            mandatory=True,
        ),
        in_bvec=dict(
            argstr="--bvecs=%s",
            extensions=None,
            mandatory=True,
        ),
        in_file=dict(
            argstr="--imain=%s",
            extensions=None,
            mandatory=True,
        ),
        in_index=dict(
            argstr="--index=%s",
            extensions=None,
            mandatory=True,
        ),
        in_mask=dict(
            argstr="--mask=%s",
            extensions=None,
            mandatory=True,
        ),
        in_topup_fieldcoef=dict(
            argstr="--topup=%s",
            extensions=None,
            requires=["in_topup_movpar"],
        ),
        in_topup_movpar=dict(
            extensions=None,
            requires=["in_topup_fieldcoef"],
        ),
        initrand=dict(
            argstr="--initrand",
            min_ver="5.0.10",
        ),
        interp=dict(
            argstr="--interp=%s",
            usedefault=True,
        ),
        is_shelled=dict(
            argstr="--data_is_shelled",
        ),
        json=dict(
            argstr="--json=%s",
            min_ver="6.0.1",
            requires=["mporder"],
            xor=["slice_order"],
        ),
        mbs_ksp=dict(
            argstr="--mbs_ksp=%smm",
            min_ver="6.0.1",
            requires=["estimate_move_by_susceptibility"],
        ),
        mbs_lambda=dict(
            argstr="--mbs_lambda=%s",
            min_ver="6.0.1",
            requires=["estimate_move_by_susceptibility"],
        ),
        mbs_niter=dict(
            argstr="--mbs_niter=%s",
            min_ver="6.0.1",
            requires=["estimate_move_by_susceptibility"],
        ),
        method=dict(
            argstr="--resamp=%s",
            usedefault=True,
        ),
        mporder=dict(
            argstr="--mporder=%s",
            min_ver="5.0.11",
            requires=["use_cuda"],
        ),
        multiband_factor=dict(
            argstr="--mb=%s",
            min_ver="5.0.10",
        ),
        multiband_offset=dict(
            argstr="--mb_offs=%d",
            min_ver="5.0.10",
            requires=["multiband_factor"],
        ),
        niter=dict(
            argstr="--niter=%s",
            usedefault=True,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        nvoxhp=dict(
            argstr="--nvoxhp=%s",
            usedefault=True,
        ),
        out_base=dict(
            argstr="--out=%s",
            usedefault=True,
        ),
        outlier_nstd=dict(
            argstr="--ol_nstd",
            min_ver="5.0.10",
            requires=["repol"],
        ),
        outlier_nvox=dict(
            argstr="--ol_nvox",
            min_ver="5.0.10",
            requires=["repol"],
        ),
        outlier_pos=dict(
            argstr="--ol_pos",
            min_ver="5.0.10",
            requires=["repol"],
        ),
        outlier_sqr=dict(
            argstr="--ol_sqr",
            min_ver="5.0.10",
            requires=["repol"],
        ),
        outlier_type=dict(
            argstr="--ol_type",
            min_ver="5.0.10",
            requires=["repol"],
        ),
        output_type=dict(),
        repol=dict(
            argstr="--repol",
        ),
        residuals=dict(
            argstr="--residuals",
            min_ver="5.0.10",
        ),
        session=dict(
            argstr="--session=%s",
            extensions=None,
        ),
        slice2vol_interp=dict(
            argstr="--s2v_interp=%s",
            min_ver="5.0.11",
            requires=["mporder"],
        ),
        slice2vol_lambda=dict(
            argstr="--s2v_lambda=%d",
            min_ver="5.0.11",
            requires=["mporder"],
        ),
        slice2vol_niter=dict(
            argstr="--s2v_niter=%d",
            min_ver="5.0.11",
            requires=["mporder"],
        ),
        slice_order=dict(
            argstr="--slspec=%s",
            min_ver="5.0.11",
            requires=["mporder"],
            xor=["json"],
        ),
        slm=dict(
            argstr="--slm=%s",
            usedefault=True,
        ),
        use_cuda=dict(),
    )
    inputs = Eddy.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Eddy_outputs():
    output_map = dict(
        out_cnr_maps=dict(
            extensions=None,
        ),
        out_corrected=dict(
            extensions=None,
        ),
        out_movement_over_time=dict(
            extensions=None,
        ),
        out_movement_rms=dict(
            extensions=None,
        ),
        out_outlier_free=dict(
            extensions=None,
        ),
        out_outlier_map=dict(
            extensions=None,
        ),
        out_outlier_n_sqr_stdev_map=dict(
            extensions=None,
        ),
        out_outlier_n_stdev_map=dict(
            extensions=None,
        ),
        out_outlier_report=dict(
            extensions=None,
        ),
        out_parameter=dict(
            extensions=None,
        ),
        out_residuals=dict(
            extensions=None,
        ),
        out_restricted_movement_rms=dict(
            extensions=None,
        ),
        out_rotated_bvecs=dict(
            extensions=None,
        ),
        out_shell_alignment_parameters=dict(
            extensions=None,
        ),
        out_shell_pe_translation_parameters=dict(
            extensions=None,
        ),
    )
    outputs = Eddy.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
""" Test the nipype interface caching mechanism
"""

from .. import Memory
from ...pipeline.engine.tests.test_engine import EngineTestInterface

from ... import config

config.set_default_config()

nb_runs = 0


class SideEffectInterface(EngineTestInterface):
    def _run_interface(self, runtime):
        global nb_runs
        nb_runs += 1
        return super(SideEffectInterface, self)._run_interface(runtime)


def test_caching(tmpdir):
    old_rerun = config.get("execution", "stop_on_first_rerun")
    try:
        # Prevent rerun to check that evaluation is computed only once
        config.set("execution", "stop_on_first_rerun", "true")
        mem = Memory(tmpdir.strpath)
        first_nb_run = nb_runs
        results = mem.cache(SideEffectInterface)(input1=2, input2=1)
        assert nb_runs == first_nb_run + 1
        assert results.outputs.output1 == [1, 2]
        results = mem.cache(SideEffectInterface)(input1=2, input2=1)
        # Check that the node hasn't been rerun
        assert nb_runs == first_nb_run + 1
        assert results.outputs.output1 == [1, 2]
        results = mem.cache(SideEffectInterface)(input1=1, input2=1)
        # Check that the node hasn been rerun
        assert nb_runs == first_nb_run + 2
        assert results.outputs.output1 == [1, 1]
    finally:
        config.set("execution", "stop_on_first_rerun", old_rerun)



# -*- coding: utf-8 -*-
from .brains import (
    BRAINSConstellationModeler,
    landmarksConstellationWeights,
    BRAINSTrimForegroundInDirection,
    BRAINSLmkTransform,
    BRAINSMush,
    BRAINSTransformConvert,
    landmarksConstellationAligner,
    BRAINSEyeDetector,
    BRAINSLinearModelerEPCA,
    BRAINSInitializedControlPoints,
    CleanUpOverlapLabels,
    BRAINSClipInferior,
    GenerateLabelMapFromProbabilityMap,
    BRAINSAlignMSP,
    BRAINSLandmarkInitializer,
    insertMidACPCpoint,
    BRAINSSnapShotWriter,
    JointHistogram,
    ShuffleVectorsModule,
    ImageRegionPlotter,
)



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..segmentation import JointFusion


def test_JointFusion_inputs():
    input_map = dict(
        alpha=dict(
            argstr="-a %s",
            usedefault=True,
        ),
        args=dict(
            argstr="%s",
        ),
        atlas_image=dict(
            argstr="-g %s...",
            mandatory=True,
        ),
        atlas_segmentation_image=dict(
            argstr="-l %s...",
            mandatory=True,
        ),
        beta=dict(
            argstr="-b %s",
            usedefault=True,
        ),
        constrain_nonnegative=dict(
            argstr="-c",
            usedefault=True,
        ),
        dimension=dict(
            argstr="-d %d",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        exclusion_image=dict(),
        exclusion_image_label=dict(
            argstr="-e %s",
            requires=["exclusion_image"],
        ),
        mask_image=dict(
            argstr="-x %s",
            extensions=None,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_atlas_voting_weight_name_format=dict(
            requires=[
                "out_label_fusion",
                "out_intensity_fusion_name_format",
                "out_label_post_prob_name_format",
            ],
        ),
        out_intensity_fusion_name_format=dict(
            argstr="",
        ),
        out_label_fusion=dict(
            argstr="%s",
            extensions=None,
            hash_files=False,
        ),
        out_label_post_prob_name_format=dict(
            requires=["out_label_fusion", "out_intensity_fusion_name_format"],
        ),
        patch_metric=dict(
            argstr="-m %s",
        ),
        patch_radius=dict(
            argstr="-p %s",
            maxlen=3,
            minlen=3,
        ),
        retain_atlas_voting_images=dict(
            argstr="-f",
            usedefault=True,
        ),
        retain_label_posterior_images=dict(
            argstr="-r",
            requires=["atlas_segmentation_image"],
            usedefault=True,
        ),
        search_radius=dict(
            argstr="-s %s",
            usedefault=True,
        ),
        target_image=dict(
            argstr="-t %s",
            mandatory=True,
        ),
        verbose=dict(
            argstr="-v",
        ),
    )
    inputs = JointFusion.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_JointFusion_outputs():
    output_map = dict(
        out_atlas_voting_weight=dict(),
        out_intensity_fusion=dict(),
        out_label_fusion=dict(
            extensions=None,
        ),
        out_label_post_prob=dict(),
    )
    outputs = JointFusion.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..r import RCommand


def test_RCommand_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        rfile=dict(
            usedefault=True,
        ),
        script=dict(
            argstr='-e "%s"',
            mandatory=True,
            position=-1,
        ),
        script_file=dict(
            extensions=None,
            usedefault=True,
        ),
    )
    inputs = RCommand.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import os

import pytest
import nipype.interfaces.fsl.model as fsl
from nipype.interfaces.fsl import no_fsl
from pathlib import Path
from ....pipeline import engine as pe


@pytest.mark.skipif(no_fsl(), reason="fsl is not installed")
def test_MultipleRegressDesign(tmpdir):
    designer = pe.Node(
        fsl.MultipleRegressDesign(), name="designer", base_dir=str(tmpdir)
    )
    designer.inputs.regressors = dict(
        voice_stenght=[1, 1, 1], age=[0.2, 0.4, 0.5], BMI=[1, -1, 2]
    )
    con1 = ["voice_and_age", "T", ["age", "voice_stenght"], [0.5, 0.5]]
    con2 = ["just_BMI", "T", ["BMI"], [1]]
    designer.inputs.contrasts = [
        con1,
        con2,
        ["con3", "F", [con1, con2]],
        ["con4", "F", [con2]],
    ]
    res = designer.run()
    outputs = res.outputs.get_traitsfree()

    for ftype in ["mat", "con", "fts", "grp"]:
        assert Path(outputs["design_" + ftype]).exists()

    expected_content = {}

    expected_content[
        "design_mat"
    ] = """/NumWaves       3
/NumPoints      3
/PPheights      3.000000e+00 5.000000e-01 1.000000e+00

/Matrix
1.000000e+00 2.000000e-01 1.000000e+00
-1.000000e+00 4.000000e-01 1.000000e+00
2.000000e+00 5.000000e-01 1.000000e+00
"""

    expected_content[
        "design_con"
    ] = """/ContrastName1   voice_and_age
/ContrastName2   just_BMI
/NumWaves       3
/NumContrasts   2
/PPheights          1.000000e+00 1.000000e+00
/RequiredEffect     100.000 100.000

/Matrix
0.000000e+00 5.000000e-01 5.000000e-01
1.000000e+00 0.000000e+00 0.000000e+00
"""

    expected_content[
        "design_fts"
    ] = """/NumWaves       2
/NumContrasts   2

/Matrix
1 1
0 1
"""

    expected_content[
        "design_grp"
    ] = """/NumWaves       1
/NumPoints      3

/Matrix
1
1
1
"""
    for ftype in ["mat", "con", "fts", "grp"]:
        outfile = "design_" + ftype
        assert Path(outputs[outfile]).read_text() == expected_content[outfile]



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..model import Deconvolve


def test_Deconvolve_inputs():
    input_map = dict(
        STATmask=dict(
            argstr="-STATmask %s",
            extensions=None,
        ),
        TR_1D=dict(
            argstr="-TR_1D %f",
        ),
        allzero_OK=dict(
            argstr="-allzero_OK",
        ),
        args=dict(
            argstr="%s",
        ),
        automask=dict(
            argstr="-automask",
        ),
        cbucket=dict(
            argstr="-cbucket %s",
        ),
        censor=dict(
            argstr="-censor %s",
            extensions=None,
        ),
        dmbase=dict(
            argstr="-dmbase",
        ),
        dname=dict(
            argstr="-D%s=%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        force_TR=dict(
            argstr="-force_TR %f",
            position=0,
        ),
        fout=dict(
            argstr="-fout",
        ),
        global_times=dict(
            argstr="-global_times",
            xor=["local_times"],
        ),
        glt_label=dict(
            argstr="-glt_label %d %s...",
            position=-1,
            requires=["gltsym"],
        ),
        gltsym=dict(
            argstr="-gltsym 'SYM: %s'...",
            position=-2,
        ),
        goforit=dict(
            argstr="-GOFORIT %i",
        ),
        in_files=dict(
            argstr="-input %s",
            copyfile=False,
            position=1,
            sep=" ",
        ),
        input1D=dict(
            argstr="-input1D %s",
            extensions=None,
        ),
        legendre=dict(
            argstr="-legendre",
        ),
        local_times=dict(
            argstr="-local_times",
            xor=["global_times"],
        ),
        mask=dict(
            argstr="-mask %s",
            extensions=None,
        ),
        noblock=dict(
            argstr="-noblock",
        ),
        nocond=dict(
            argstr="-nocond",
        ),
        nodmbase=dict(
            argstr="-nodmbase",
        ),
        nofdr=dict(
            argstr="-noFDR",
        ),
        nolegendre=dict(
            argstr="-nolegendre",
        ),
        nosvd=dict(
            argstr="-nosvd",
        ),
        num_glt=dict(
            argstr="-num_glt %d",
            position=-3,
        ),
        num_stimts=dict(
            argstr="-num_stimts %d",
            position=-6,
        ),
        num_threads=dict(
            argstr="-jobs %d",
            nohash=True,
        ),
        ortvec=dict(
            argstr="-ortvec %s %s",
        ),
        out_file=dict(
            argstr="-bucket %s",
            extensions=None,
        ),
        outputtype=dict(),
        polort=dict(
            argstr="-polort %d",
        ),
        rmsmin=dict(
            argstr="-rmsmin %f",
        ),
        rout=dict(
            argstr="-rout",
        ),
        sat=dict(
            argstr="-sat",
            xor=["trans"],
        ),
        singvals=dict(
            argstr="-singvals",
        ),
        stim_label=dict(
            argstr="-stim_label %d %s...",
            position=-4,
            requires=["stim_times"],
        ),
        stim_times=dict(
            argstr="-stim_times %d %s '%s'...",
            position=-5,
        ),
        stim_times_subtract=dict(
            argstr="-stim_times_subtract %f",
        ),
        svd=dict(
            argstr="-svd",
        ),
        tout=dict(
            argstr="-tout",
        ),
        trans=dict(
            argstr="-trans",
            xor=["sat"],
        ),
        vout=dict(
            argstr="-vout",
        ),
        x1D=dict(
            argstr="-x1D %s",
            extensions=None,
        ),
        x1D_stop=dict(
            argstr="-x1D_stop",
        ),
    )
    inputs = Deconvolve.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_Deconvolve_outputs():
    output_map = dict(
        cbucket=dict(
            extensions=None,
        ),
        out_file=dict(
            extensions=None,
        ),
        reml_script=dict(
            extensions=None,
        ),
        x1D=dict(
            extensions=None,
        ),
    )
    outputs = Deconvolve.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..meshfix import MeshFix


def test_MeshFix_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        cut_inner=dict(
            argstr="--cut-inner %d",
        ),
        cut_outer=dict(
            argstr="--cut-outer %d",
        ),
        decouple_inin=dict(
            argstr="--decouple-inin %d",
        ),
        decouple_outin=dict(
            argstr="--decouple-outin %d",
        ),
        decouple_outout=dict(
            argstr="--decouple-outout %d",
        ),
        dilation=dict(
            argstr="--dilate %d",
        ),
        dont_clean=dict(
            argstr="--no-clean",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        epsilon_angle=dict(
            argstr="-a %f",
        ),
        finetuning_distance=dict(
            argstr="%f",
            position=-2,
            requires=["finetuning_substeps"],
        ),
        finetuning_inwards=dict(
            argstr="--fineTuneIn ",
            position=-3,
            requires=["finetuning_distance", "finetuning_substeps"],
        ),
        finetuning_outwards=dict(
            argstr="--fineTuneOut ",
            position=-3,
            requires=["finetuning_distance", "finetuning_substeps"],
            xor=["finetuning_inwards"],
        ),
        finetuning_substeps=dict(
            argstr="%d",
            position=-1,
            requires=["finetuning_distance"],
        ),
        in_file1=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=1,
        ),
        in_file2=dict(
            argstr="%s",
            extensions=None,
            position=2,
        ),
        join_closest_components=dict(
            argstr="-jc",
            xor=["join_closest_components"],
        ),
        join_overlapping_largest_components=dict(
            argstr="-j",
            xor=["join_closest_components"],
        ),
        laplacian_smoothing_steps=dict(
            argstr="--smooth %d",
        ),
        number_of_biggest_shells=dict(
            argstr="--shells %d",
        ),
        out_filename=dict(
            argstr="-o %s",
            extensions=None,
            genfile=True,
        ),
        output_type=dict(
            usedefault=True,
        ),
        quiet_mode=dict(
            argstr="-q",
        ),
        remove_handles=dict(
            argstr="--remove-handles",
        ),
        save_as_freesurfer_mesh=dict(
            argstr="--fsmesh",
            xor=["save_as_vrml", "save_as_stl"],
        ),
        save_as_stl=dict(
            argstr="--stl",
            xor=["save_as_vrml", "save_as_freesurfer_mesh"],
        ),
        save_as_vrml=dict(
            argstr="--wrl",
            xor=["save_as_stl", "save_as_freesurfer_mesh"],
        ),
        set_intersections_to_one=dict(
            argstr="--intersect",
        ),
        uniform_remeshing_steps=dict(
            argstr="-u %d",
            requires=["uniform_remeshing_vertices"],
        ),
        uniform_remeshing_vertices=dict(
            argstr="--vertices %d",
            requires=["uniform_remeshing_steps"],
        ),
        x_shift=dict(
            argstr="--smooth %d",
        ),
    )
    inputs = MeshFix.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_MeshFix_outputs():
    output_map = dict(
        mesh_file=dict(
            extensions=None,
        ),
    )
    outputs = MeshFix.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..visualization import CreateTiledMosaic


def test_CreateTiledMosaic_inputs():
    input_map = dict(
        alpha_value=dict(
            argstr="-a %.2f",
        ),
        args=dict(
            argstr="%s",
        ),
        direction=dict(
            argstr="-d %d",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        flip_slice=dict(
            argstr="-f %s",
        ),
        input_image=dict(
            argstr="-i %s",
            extensions=None,
            mandatory=True,
        ),
        mask_image=dict(
            argstr="-x %s",
            extensions=None,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        output_image=dict(
            argstr="-o %s",
            usedefault=True,
        ),
        pad_or_crop=dict(
            argstr="-p %s",
        ),
        permute_axes=dict(
            argstr="-g",
        ),
        rgb_image=dict(
            argstr="-r %s",
            extensions=None,
            mandatory=True,
        ),
        slices=dict(
            argstr="-s %s",
        ),
        tile_geometry=dict(
            argstr="-t %s",
        ),
    )
    inputs = CreateTiledMosaic.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_CreateTiledMosaic_outputs():
    output_map = dict(
        output_image=dict(
            extensions=None,
        ),
    )
    outputs = CreateTiledMosaic.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..registration import AnalyzeWarp


def test_AnalyzeWarp_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        jac=dict(
            argstr="-jac %s",
            usedefault=True,
        ),
        jacmat=dict(
            argstr="-jacmat %s",
            usedefault=True,
        ),
        moving_image=dict(
            argstr="-in %s",
            extensions=None,
        ),
        num_threads=dict(
            argstr="-threads %01d",
            nohash=True,
            usedefault=True,
        ),
        output_path=dict(
            argstr="-out %s",
            mandatory=True,
            usedefault=True,
        ),
        points=dict(
            argstr="-def %s",
            position=0,
            usedefault=True,
        ),
        transform_file=dict(
            argstr="-tp %s",
            extensions=None,
            mandatory=True,
        ),
    )
    inputs = AnalyzeWarp.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_AnalyzeWarp_outputs():
    output_map = dict(
        disp_field=dict(
            extensions=None,
        ),
        jacdet_map=dict(
            extensions=None,
        ),
        jacmat_map=dict(
            extensions=None,
        ),
    )
    outputs = AnalyzeWarp.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
